#include <algorithm>
#include <cmath>
#include <cstddef>
#include <memory>
#include <arbor/mechanism_abi.h>
#include <arbor/math.hpp>

namespace arb {
namespace allen_catalogue {
namespace kernel_NaV {

using ::arb::math::exprelr;
using ::arb::math::safeinv;
using ::std::abs;
using ::std::cos;
using ::std::exp;
using ::std::max;
using ::std::min;
using ::std::pow;
using ::std::sin;
using ::std::sqrt;
using ::std::tanh;

static constexpr unsigned simd_width_ = 1;
static constexpr unsigned min_align_ = std::max(alignof(arb_value_type), alignof(arb_index_type));
using ::std::log;

#define PPACK_IFACE_BLOCK \
[[maybe_unused]] auto _pp_var_width                                                 = pp->width;\
[[maybe_unused]] auto _pp_var_n_detectors                                           = pp->n_detectors;\
[[maybe_unused]] auto _pp_var_dt                                                    = pp->dt;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_vec_ci                       = pp->vec_ci;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_v                        = pp->vec_v;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_i                        = pp->vec_i;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_g                        = pp->vec_g;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_temperature_degC             = pp->temperature_degC;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_diam_um                      = pp->diam_um;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_area_um2                     = pp->area_um2;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_time_since_spike             = pp->time_since_spike;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_node_index                   = pp->node_index;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_peer_index                   = pp->peer_index;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_multiplicity                 = pp->multiplicity;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_weight                       = pp->weight;\
[[maybe_unused]] auto& _pp_var_events                                               = pp->events;\
[[maybe_unused]] auto _pp_var_mechanism_id                                          = pp->mechanism_id;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_contiguous               = pp->index_constraints.n_contiguous;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_constant                 = pp->index_constraints.n_constant;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_independent              = pp->index_constraints.n_independent;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_none                     = pp->index_constraints.n_none;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_contiguous  = pp->index_constraints.contiguous;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_constant    = pp->index_constraints.constant;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_independent = pp->index_constraints.independent;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_none        = pp->index_constraints.none;\
[[maybe_unused]] auto _pp_var_Con = pp->globals[0];\
[[maybe_unused]] auto _pp_var_Coff = pp->globals[1];\
[[maybe_unused]] auto _pp_var_Oon = pp->globals[2];\
[[maybe_unused]] auto _pp_var_Ooff = pp->globals[3];\
[[maybe_unused]] auto _pp_var_alpha = pp->globals[4];\
[[maybe_unused]] auto _pp_var_beta = pp->globals[5];\
[[maybe_unused]] auto _pp_var_gamma = pp->globals[6];\
[[maybe_unused]] auto _pp_var_delta = pp->globals[7];\
[[maybe_unused]] auto _pp_var_alfac = pp->globals[8];\
[[maybe_unused]] auto _pp_var_btfac = pp->globals[9];\
[[maybe_unused]] auto _pp_var_x1 = pp->globals[10];\
[[maybe_unused]] auto _pp_var_x2 = pp->globals[11];\
[[maybe_unused]] auto const * const * _pp_var_random_numbers = pp->random_numbers;\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_C1 = pp->state_vars[0];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_C2 = pp->state_vars[1];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_C3 = pp->state_vars[2];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_C4 = pp->state_vars[3];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_C5 = pp->state_vars[4];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I1 = pp->state_vars[5];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I2 = pp->state_vars[6];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I3 = pp->state_vars[7];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I4 = pp->state_vars[8];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I5 = pp->state_vars[9];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_O = pp->state_vars[10];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_I6 = pp->state_vars[11];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_qt = pp->state_vars[12];\
[[maybe_unused]] arb_value_type* __restrict__ _pp_var_gbar = pp->parameters[0];\
[[maybe_unused]] auto& _pp_var_ion_na = pp->ion_states[0];\
[[maybe_unused]] auto* __restrict__ _pp_var_ion_na_index = pp->ion_states[0].index;\
//End of IFACEBLOCK


// interface methods
static void init(arb_mechanism_ppack* pp) {
    PPACK_IFACE_BLOCK;
    for (arb_size_type i_ = 0; i_ < _pp_var_width; ++i_) {
        auto node_indexi_ = _pp_var_node_index[i_];
        arb_value_type celsius = _pp_var_temperature_degC[node_indexi_];
        arb_value_type v = _pp_var_vec_v[node_indexi_];
        arb_value_type inv_73_, inv_72_, t_318_, t_317_, t_316_, t_313_, t_311_, t_307_, t_306_, inv_64_, t_305_, inv_63_, t_303_, t_299_, t_297_, t_295_, inv_68_, t_292_, t_291_, t_290_, t_286_, t_284_, inv_71_, t_282_, t_280_, t_279_, t_278_, inv_54_, t_275_, t_274_, t_273_, inv_65_, t_272_, inv_51_, t_269_, t_264_, inv_49_, t_261_, t_260_, t_259_, inv_48_, t_257_, t_289_, t_256_, t_254_, t_293_, inv_47_, t_253_, t_250_, t_249_, inv_46_, t_246_, t_245_, t_301_, inv_44_, t_240_, t_239_, inv_43_, t_238_, t_237_, t_236_, inv_42_, t_233_, t_232_, inv_41_, t_231_, t_229_, t_228_, inv_40_, t_271_, t_226_, t_224_, t_220_, t_300_, t_225_, t_219_, inv_62_, t_217_, t_216_, t_215_, inv_37_, t_213_, t_211_, t_210_, t_209_, t_208_, inv_35_, t_203_, t_281_, t_268_, t_202_, t_201_, inv_34_, t_200_, t_199_, t_198_, inv_33_, t_193_, t_192_, t_191_, t_188_, t_258_, t_187_, t_186_, t_183_, t_235_, t_182_, t_181_, t_204_, inv_30_, t_322_, t_180_, inv_55_, t_175_, t_173_, t_172_, t_170_, t_166_, t_165_, t_168_, inv_27_, t_162_, t_160_, t_159_, inv_31_, inv_26_, t_158_, t_157_, inv_52_, t_155_, t_154_, t_265_, inv_25_, t_234_, t_152_, inv_60_, t_151_, t_177_, t_149_, t_148_, t_147_, inv_24_, t_145_, inv_28_, t_144_, t_143_, t_140_, t_136_, l_55_, t_251_, t_176_, t_56_, t_167_, t_24_, t_21_, t_319_, t_270_, t_19_, inv_6_, t_102_, t_298_, t_242_, t_18_, l_25_, t_304_, inv_10_, t_132_, t_13_, bi2, t_12_, t_302_, t_10_, t_223_, t_9_, t_294_, inv_59_, inv_1_, bin, t_150_, t_0_, t_205_, l_64_, t_3_, l_60_, t_88_, l_58_, l_57_, fi3, t_184_, t_146_, t_127_, l_54_, l_49_, t_90_, t_14_, l_46_, t_83_, l_43_, l_42_, t_76_, t_139_, t_110_, t_314_, t_8_, t_247_, l_39_, inv_2_, t_59_, t_288_, t_111_, l_44_, t_11_, l_50_, l_37_, l_34_, f13, inv_3_, t_23_, l_33_, l_45_, t_58_, l_32_, fi5, t_214_, l_30_, l_29_, t_255_, l_28_, t_221_, l_53_, inv_50_, l_22_, l_10_, l_63_, inv_70_, t_262_, b0O, inv_39_, l_59_, t_61_, t_72_, t_99_, t_128_, l_27_, f04, l_31_, b14, f02, t_207_, t_7_, t_194_, l_61_, t_25_, t_36_, t_80_, b11, f11, inv_58_, t_41_, l_12_, l_51_, t_82_, t_38_, l_48_, b02, inv_45_, t_212_, t_73_, inv_29_, bi1, l_4_, inv_66_, t_22_, l_17_, b01, t_222_, t_116_, t_308_, t_125_, l_14_, t_84_, t_124_, f01, t_178_, l_20_, t_164_, t_135_, l_15_, l_35_, t_252_, t_94_, fin, b13, t_263_, l_47_, t_312_, t_35_, inv_17_, inv_38_, b12, f03, l_11_, l_40_, b04, bi5, l_26_, bi3, t_227_, t_91_, inv_69_, l_5_, t_4_, inv_53_, f14, l_16_, inv_57_, l_41_, fi1, t_190_, l_8_, t_75_, t_86_, t_197_, l_1_, t_117_, t_1_, fi4, t_315_, t_206_, t_51_, l_2_, t_31_, t_30_, l_3_, ibtf, l_6_, t_266_, t_53_, t_103_, t_109_, l_56_, l_7_, t_65_, t_283_, l_9_, l_0_, t_218_, inv_36_, t_161_, t_16_, t_15_, t_66_, bi4, t_118_, inv_61_, l_13_, l_18_, l_19_, l_62_, l_23_, l_36_, t_230_, inv_15_, inv_56_, fi2, inv_8_, t_26_, t_310_, t_156_, t_28_, t_6_, inv_4_, t_142_, t_5_, t_29_, inv_5_, t_71_, t_287_, t_123_, t_34_, t_37_, t_137_, t_42_, t_33_, t_50_, t_44_, t_46_, f0O, inv_7_, t_296_, t_277_, t_171_, t_17_, t_45_, t_48_, t_85_, t_49_, inv_67_, t_179_, t_52_, t_54_, t_285_, t_55_, t_113_, t_243_, t_32_, t_131_, t_323_, l_21_, t_133_, t_309_, t_89_, t_62_, t_63_, t_64_, t_196_, inv_9_, t_67_, inv_32_, t_92_, t_68_, t_95_, t_57_, inv_0_, inv_14_, b03, inv_16_, t_69_, t_153_, t_74_, t_100_, t_163_, t_2_, inv_11_, t_195_, t_141_, t_47_, t_77_, t_78_, t_70_, t_79_, inv_12_, l_38_, t_87_, t_60_, inv_19_, t_276_, t_244_, inv_13_, t_267_, t_93_, l_52_, t_96_, t_138_, t_97_, t_81_, t_98_, t_27_, t_101_, t_104_, inv_23_, l_24_, t_108_, t_43_, t_105_, t_169_, inv_18_, t_106_, t_107_, t_241_, t_112_, t_174_, t_39_, t_114_, t_115_, t_185_, t_20_, t_119_, t_120_, t_121_, t_122_, t_126_, t_40_, inv_20_, t_321_, t_320_, t_189_, t_129_, t_248_, t_130_, inv_21_, t_134_, f12, inv_22_;
        _pp_var_qt[i_] = exp( 0.83290912293510388*((celsius- 37.0)* 0.10000000000000001));
        ibtf =  1.0/_pp_var_btfac;
        f04 = _pp_var_qt[i_]*_pp_var_alpha*exp(v/_pp_var_x1);
        f03 =  2.0*f04;
        f02 =  3.0*f04;
        f01 =  4.0*f04;
        f0O = _pp_var_qt[i_]*_pp_var_gamma;
        f14 = _pp_var_alfac*f04;
        f13 =  2.0*f14;
        f12 =  3.0*f14;
        f11 =  4.0*f14;
        fi1 = _pp_var_qt[i_]*_pp_var_Con;
        fi2 = fi1*_pp_var_alfac;
        fi3 = fi2*_pp_var_alfac;
        fi4 = fi3*_pp_var_alfac;
        fi5 = fi4*_pp_var_alfac;
        fin = _pp_var_qt[i_]*_pp_var_Oon;
        b01 = _pp_var_qt[i_]*_pp_var_beta*exp(v/_pp_var_x2);
        b02 =  2.0*b01;
        b03 =  3.0*b01;
        b04 =  4.0*b01;
        b0O = _pp_var_qt[i_]*_pp_var_delta;
        b11 = b01*ibtf;
        b12 =  2.0*b11;
        b13 =  3.0*b11;
        b14 =  4.0*b11;
        bi1 = _pp_var_qt[i_]*_pp_var_Coff;
        bi2 = bi1*ibtf;
        bi3 = bi2*ibtf;
        bi4 = bi3*ibtf;
        bi5 = bi4*ibtf;
        bin = _pp_var_qt[i_]*_pp_var_Ooff;
        l_0_ =  -(fi1+f01);
        l_1_ = b01;
        l_2_ = bi1;
        l_3_ =  0.;
        l_4_ = f01;
        l_5_ = bi2;
        l_6_ =  -(b01+fi2+f02);
        l_7_ = b02;
        l_8_ =  0.;
        l_9_ = bi3;
        l_10_ = f02;
        l_11_ = b03;
        l_12_ =  -(b02+fi3+f03);
        l_13_ =  0.;
        l_14_ = b04;
        l_15_ =  -(b03+fi4+f04);
        l_16_ = f03;
        l_17_ = bi4;
        l_18_ =  0.;
        l_19_ = bi5;
        l_20_ =  -(b04+fi5+f0O);
        l_21_ = f04;
        l_22_ = b0O;
        l_23_ =  0.;
        l_24_ = f0O;
        l_25_ =  -(b0O+fin);
        l_26_ = bin;
        l_27_ =  0.;
        l_28_ = fi1;
        l_29_ = b11;
        l_30_ =  -(bi1+f11);
        l_31_ =  0.;
        l_32_ = b12;
        l_33_ =  -(b11+bi2+f12);
        l_34_ = fi2;
        l_35_ = f11;
        l_36_ =  0.;
        l_37_ =  -(b12+bi3+f13);
        l_38_ = f12;
        l_39_ = fi3;
        l_40_ = bi3;
        l_41_ =  0.;
        l_42_ = f13;
        l_43_ = b14;
        l_44_ = fi4;
        l_45_ =  -(b13+bi4+f14);
        l_46_ =  0.;
        l_47_ =  -(b14+bi5+f0O);
        l_48_ = fi5;
        l_49_ = f14;
        l_50_ = b0O;
        l_51_ =  0.;
        l_52_ =  1.0;
        l_53_ =  1.0;
        l_54_ =  1.0;
        l_55_ =  1.0;
        l_56_ =  1.0;
        l_57_ =  1.0;
        l_58_ =  1.0;
        l_59_ =  1.0;
        l_60_ =  1.0;
        l_61_ =  1.0;
        l_62_ =  1.0;
        l_63_ =  1.0;
        l_64_ =  1.0;
        t_0_ = l_30_*l_0_-l_2_*l_28_;
        t_1_ =  -(l_2_*l_29_);
        t_2_ = l_30_*l_1_;
        t_3_ = l_30_*l_3_-l_2_*l_31_;
        inv_0_ = safeinv(max(max(max(abs(t_0_), abs(t_1_)), abs(t_2_)), abs(t_3_)));
        t_0_ = t_0_*inv_0_;
        t_1_ = t_1_*inv_0_;
        t_2_ = t_2_*inv_0_;
        t_3_ = t_3_*inv_0_;
        t_4_ = l_30_*l_32_;
        t_5_ =  -(l_35_*l_28_);
        t_6_ = l_30_*l_33_-l_35_*l_29_;
        t_7_ = l_30_*l_34_;
        t_8_ = l_30_*l_36_-l_35_*l_31_;
        inv_1_ = safeinv(max(max(max(max(abs(t_4_), abs(t_5_)), abs(t_6_)), abs(t_7_)), abs(t_8_)));
        t_4_ = t_4_*inv_1_;
        t_5_ = t_5_*inv_1_;
        t_6_ = t_6_*inv_1_;
        t_7_ = t_7_*inv_1_;
        t_8_ = t_8_*inv_1_;
        t_9_ = l_30_*l_52_;
        t_10_ = l_30_*l_53_;
        t_11_ = l_30_*l_54_;
        t_12_ = l_30_*l_55_-l_58_*l_28_;
        t_13_ = l_30_*l_56_-l_58_*l_29_;
        t_14_ = l_30_*l_57_;
        t_15_ = l_30_*l_59_;
        t_16_ = l_30_*l_60_;
        t_17_ = l_30_*l_61_;
        t_18_ = l_30_*l_62_;
        t_19_ = l_30_*l_63_;
        t_20_ = l_30_*l_64_-l_58_*l_31_;
        inv_2_ = safeinv(max(max(max(max(max(max(max(max(max(max(max(abs(t_9_), abs(t_10_)), abs(t_11_)), abs(t_12_)), abs(t_13_)), abs(t_14_)), abs(t_15_)), abs(t_16_)), abs(t_17_)), abs(t_18_)), abs(t_19_)), abs(t_20_)));
        t_9_ = t_9_*inv_2_;
        t_10_ = t_10_*inv_2_;
        t_11_ = t_11_*inv_2_;
        t_12_ = t_12_*inv_2_;
        t_13_ = t_13_*inv_2_;
        t_14_ = t_14_*inv_2_;
        t_15_ = t_15_*inv_2_;
        t_16_ = t_16_*inv_2_;
        t_17_ = t_17_*inv_2_;
        t_18_ = t_18_*inv_2_;
        t_19_ = t_19_*inv_2_;
        t_20_ = t_20_*inv_2_;
        t_21_ = t_0_*l_5_-l_4_*t_1_;
        t_22_ = t_0_*l_6_-l_4_*t_2_;
        t_23_ = t_0_*l_7_;
        t_24_ = t_0_*l_8_-l_4_*t_3_;
        inv_3_ = safeinv(max(max(max(abs(t_21_), abs(t_22_)), abs(t_23_)), abs(t_24_)));
        t_21_ = t_21_*inv_3_;
        t_22_ = t_22_*inv_3_;
        t_23_ = t_23_*inv_3_;
        t_24_ = t_24_*inv_3_;
        t_25_ = t_0_*l_29_-l_28_*t_1_;
        t_26_ =  -(l_28_*t_2_);
        t_27_ = t_0_*l_30_;
        t_28_ = t_0_*l_31_-l_28_*t_3_;
        inv_4_ = safeinv(max(max(max(abs(t_25_), abs(t_26_)), abs(t_27_)), abs(t_28_)));
        t_25_ = t_25_*inv_4_;
        t_26_ = t_26_*inv_4_;
        t_27_ = t_27_*inv_4_;
        t_28_ = t_28_*inv_4_;
        t_29_ = t_0_*t_4_;
        t_30_ = t_0_*t_6_-t_5_*t_1_;
        t_31_ = t_0_*t_7_-t_5_*t_2_;
        t_32_ = t_0_*t_8_-t_5_*t_3_;
        inv_5_ = safeinv(max(max(max(abs(t_29_), abs(t_30_)), abs(t_31_)), abs(t_32_)));
        t_29_ = t_29_*inv_5_;
        t_30_ = t_30_*inv_5_;
        t_31_ = t_31_*inv_5_;
        t_32_ = t_32_*inv_5_;
        t_33_ = t_0_*t_9_;
        t_34_ = t_0_*t_10_;
        t_35_ = t_0_*t_11_;
        t_36_ = t_0_*t_13_-t_12_*t_1_;
        t_37_ = t_0_*t_14_-t_12_*t_2_;
        t_38_ = t_0_*t_15_;
        t_39_ = t_0_*t_16_;
        t_40_ = t_0_*t_17_;
        t_41_ = t_0_*t_18_;
        t_42_ = t_0_*t_19_;
        t_43_ = t_0_*t_20_-t_12_*t_3_;
        inv_6_ = safeinv(max(max(max(max(max(max(max(max(max(max(abs(t_33_), abs(t_34_)), abs(t_35_)), abs(t_36_)), abs(t_37_)), abs(t_38_)), abs(t_39_)), abs(t_40_)), abs(t_41_)), abs(t_42_)), abs(t_43_)));
        t_33_ = t_33_*inv_6_;
        t_34_ = t_34_*inv_6_;
        t_35_ = t_35_*inv_6_;
        t_36_ = t_36_*inv_6_;
        t_37_ = t_37_*inv_6_;
        t_38_ = t_38_*inv_6_;
        t_39_ = t_39_*inv_6_;
        t_40_ = t_40_*inv_6_;
        t_41_ = t_41_*inv_6_;
        t_42_ = t_42_*inv_6_;
        t_43_ = t_43_*inv_6_;
        t_44_ = l_47_*l_20_-l_19_*l_48_;
        t_45_ = l_47_*l_21_;
        t_46_ =  -(l_19_*l_49_);
        t_47_ = l_47_*l_22_;
        t_48_ =  -(l_19_*l_50_);
        t_49_ = l_47_*l_23_-l_19_*l_51_;
        inv_7_ = safeinv(max(max(max(max(max(abs(t_44_), abs(t_45_)), abs(t_46_)), abs(t_47_)), abs(t_48_)), abs(t_49_)));
        t_44_ = t_44_*inv_7_;
        t_45_ = t_45_*inv_7_;
        t_46_ = t_46_*inv_7_;
        t_47_ = t_47_*inv_7_;
        t_48_ = t_48_*inv_7_;
        t_49_ = t_49_*inv_7_;
        t_50_ = l_47_*l_42_;
        t_51_ =  -(l_43_*l_48_);
        t_52_ = l_47_*l_44_;
        t_53_ = l_47_*l_45_-l_43_*l_49_;
        t_54_ =  -(l_43_*l_50_);
        t_55_ = l_47_*l_46_-l_43_*l_51_;
        inv_8_ = safeinv(max(max(max(max(max(abs(t_50_), abs(t_51_)), abs(t_52_)), abs(t_53_)), abs(t_54_)), abs(t_55_)));
        t_50_ = t_50_*inv_8_;
        t_51_ = t_51_*inv_8_;
        t_52_ = t_52_*inv_8_;
        t_53_ = t_53_*inv_8_;
        t_54_ = t_54_*inv_8_;
        t_55_ = t_55_*inv_8_;
        t_56_ = l_47_*t_33_;
        t_57_ = l_47_*t_35_-t_34_*l_48_;
        t_58_ = l_47_*t_36_;
        t_59_ = l_47_*t_37_;
        t_60_ = l_47_*t_38_;
        t_61_ = l_47_*t_39_;
        t_62_ = l_47_*t_40_-t_34_*l_49_;
        t_63_ = l_47_*t_41_;
        t_64_ = l_47_*t_42_-t_34_*l_50_;
        t_65_ = l_47_*t_43_-t_34_*l_51_;
        inv_9_ = safeinv(max(max(max(max(max(max(max(max(max(abs(t_56_), abs(t_57_)), abs(t_58_)), abs(t_59_)), abs(t_60_)), abs(t_61_)), abs(t_62_)), abs(t_63_)), abs(t_64_)), abs(t_65_)));
        t_56_ = t_56_*inv_9_;
        t_57_ = t_57_*inv_9_;
        t_58_ = t_58_*inv_9_;
        t_59_ = t_59_*inv_9_;
        t_60_ = t_60_*inv_9_;
        t_61_ = t_61_*inv_9_;
        t_62_ = t_62_*inv_9_;
        t_63_ = t_63_*inv_9_;
        t_64_ = t_64_*inv_9_;
        t_65_ = t_65_*inv_9_;
        t_66_ =  -(l_9_*t_30_);
        t_67_ = t_29_*l_10_-l_9_*t_31_;
        t_68_ = t_29_*l_11_;
        t_69_ = t_29_*l_12_;
        t_70_ = t_29_*l_13_-l_9_*t_32_;
        inv_10_ = safeinv(max(max(max(max(abs(t_66_), abs(t_67_)), abs(t_68_)), abs(t_69_)), abs(t_70_)));
        t_66_ = t_66_*inv_10_;
        t_67_ = t_67_*inv_10_;
        t_68_ = t_68_*inv_10_;
        t_69_ = t_69_*inv_10_;
        t_70_ = t_70_*inv_10_;
        t_71_ = t_29_*l_38_-l_37_*t_30_;
        t_72_ =  -(l_37_*t_31_);
        t_73_ = t_29_*l_39_;
        t_74_ = t_29_*l_40_;
        t_75_ = t_29_*l_41_-l_37_*t_32_;
        inv_11_ = safeinv(max(max(max(max(abs(t_71_), abs(t_72_)), abs(t_73_)), abs(t_74_)), abs(t_75_)));
        t_71_ = t_71_*inv_11_;
        t_72_ = t_72_*inv_11_;
        t_73_ = t_73_*inv_11_;
        t_74_ = t_74_*inv_11_;
        t_75_ = t_75_*inv_11_;
        t_76_ = t_29_*t_51_;
        t_77_ =  -(t_50_*t_30_);
        t_78_ =  -(t_50_*t_31_);
        t_79_ = t_29_*t_52_;
        t_80_ = t_29_*t_53_;
        t_81_ = t_29_*t_54_;
        t_82_ = t_29_*t_55_-t_50_*t_32_;
        inv_12_ = safeinv(max(max(max(max(max(max(abs(t_76_), abs(t_77_)), abs(t_78_)), abs(t_79_)), abs(t_80_)), abs(t_81_)), abs(t_82_)));
        t_76_ = t_76_*inv_12_;
        t_77_ = t_77_*inv_12_;
        t_78_ = t_78_*inv_12_;
        t_79_ = t_79_*inv_12_;
        t_80_ = t_80_*inv_12_;
        t_81_ = t_81_*inv_12_;
        t_82_ = t_82_*inv_12_;
        t_83_ = t_29_*t_57_;
        t_84_ = t_29_*t_58_-t_56_*t_30_;
        t_85_ = t_29_*t_59_-t_56_*t_31_;
        t_86_ = t_29_*t_60_;
        t_87_ = t_29_*t_61_;
        t_88_ = t_29_*t_62_;
        t_89_ = t_29_*t_63_;
        t_90_ = t_29_*t_64_;
        t_91_ = t_29_*t_65_-t_56_*t_32_;
        inv_13_ = safeinv(max(max(max(max(max(max(max(max(abs(t_83_), abs(t_84_)), abs(t_85_)), abs(t_86_)), abs(t_87_)), abs(t_88_)), abs(t_89_)), abs(t_90_)), abs(t_91_)));
        t_83_ = t_83_*inv_13_;
        t_84_ = t_84_*inv_13_;
        t_85_ = t_85_*inv_13_;
        t_86_ = t_86_*inv_13_;
        t_87_ = t_87_*inv_13_;
        t_88_ = t_88_*inv_13_;
        t_89_ = t_89_*inv_13_;
        t_90_ = t_90_*inv_13_;
        t_91_ = t_91_*inv_13_;
        t_92_ = t_73_*t_21_-t_23_*t_71_;
        t_93_ = t_73_*t_22_-t_23_*t_72_;
        t_94_ =  -(t_23_*t_74_);
        t_95_ = t_73_*t_24_-t_23_*t_75_;
        inv_14_ = safeinv(max(max(max(abs(t_92_), abs(t_93_)), abs(t_94_)), abs(t_95_)));
        t_92_ = t_92_*inv_14_;
        t_93_ = t_93_*inv_14_;
        t_94_ = t_94_*inv_14_;
        t_95_ = t_95_*inv_14_;
        t_96_ = t_73_*t_66_-t_69_*t_71_;
        t_97_ = t_73_*t_67_-t_69_*t_72_;
        t_98_ = t_73_*t_68_;
        t_99_ =  -(t_69_*t_74_);
        t_100_ = t_73_*t_70_-t_69_*t_75_;
        inv_15_ = safeinv(max(max(max(max(abs(t_96_), abs(t_97_)), abs(t_98_)), abs(t_99_)), abs(t_100_)));
        t_96_ = t_96_*inv_15_;
        t_97_ = t_97_*inv_15_;
        t_98_ = t_98_*inv_15_;
        t_99_ = t_99_*inv_15_;
        t_100_ = t_100_*inv_15_;
        t_101_ = t_73_*l_14_;
        t_102_ =  -(l_16_*t_71_);
        t_103_ =  -(l_16_*t_72_);
        t_104_ = t_73_*l_15_;
        t_105_ = t_73_*l_17_-l_16_*t_74_;
        t_106_ = t_73_*l_18_-l_16_*t_75_;
        inv_16_ = safeinv(max(max(max(max(max(abs(t_101_), abs(t_102_)), abs(t_103_)), abs(t_104_)), abs(t_105_)), abs(t_106_)));
        t_101_ = t_101_*inv_16_;
        t_102_ = t_102_*inv_16_;
        t_103_ = t_103_*inv_16_;
        t_104_ = t_104_*inv_16_;
        t_105_ = t_105_*inv_16_;
        t_106_ = t_106_*inv_16_;
        t_107_ = t_73_*t_83_;
        t_108_ = t_73_*t_84_-t_87_*t_71_;
        t_109_ = t_73_*t_85_-t_87_*t_72_;
        t_110_ = t_73_*t_86_;
        t_111_ = t_73_*t_88_-t_87_*t_74_;
        t_112_ = t_73_*t_89_;
        t_113_ = t_73_*t_90_;
        t_114_ = t_73_*t_91_-t_87_*t_75_;
        inv_17_ = safeinv(max(max(max(max(max(max(max(abs(t_107_), abs(t_108_)), abs(t_109_)), abs(t_110_)), abs(t_111_)), abs(t_112_)), abs(t_113_)), abs(t_114_)));
        t_107_ = t_107_*inv_17_;
        t_108_ = t_108_*inv_17_;
        t_109_ = t_109_*inv_17_;
        t_110_ = t_110_*inv_17_;
        t_111_ = t_111_*inv_17_;
        t_112_ = t_112_*inv_17_;
        t_113_ = t_113_*inv_17_;
        t_114_ = t_114_*inv_17_;
        t_115_ = t_92_*t_0_;
        t_116_ = t_92_*t_2_-t_1_*t_93_;
        t_117_ =  -(t_1_*t_94_);
        t_118_ = t_92_*t_3_-t_1_*t_95_;
        inv_18_ = safeinv(max(max(max(abs(t_115_), abs(t_116_)), abs(t_117_)), abs(t_118_)));
        t_115_ = t_115_*inv_18_;
        t_116_ = t_116_*inv_18_;
        t_117_ = t_117_*inv_18_;
        t_118_ = t_118_*inv_18_;
        t_119_ = t_92_*t_97_-t_96_*t_93_;
        t_120_ = t_92_*t_98_;
        t_121_ = t_92_*t_99_-t_96_*t_94_;
        t_122_ = t_92_*t_100_-t_96_*t_95_;
        inv_19_ = safeinv(max(max(max(abs(t_119_), abs(t_120_)), abs(t_121_)), abs(t_122_)));
        t_119_ = t_119_*inv_19_;
        t_120_ = t_120_*inv_19_;
        t_121_ = t_121_*inv_19_;
        t_122_ = t_122_*inv_19_;
        t_123_ = t_92_*t_101_;
        t_124_ = t_92_*t_103_-t_102_*t_93_;
        t_125_ = t_92_*t_104_;
        t_126_ = t_92_*t_105_-t_102_*t_94_;
        t_127_ = t_92_*t_106_-t_102_*t_95_;
        inv_20_ = safeinv(max(max(max(max(abs(t_123_), abs(t_124_)), abs(t_125_)), abs(t_126_)), abs(t_127_)));
        t_123_ = t_123_*inv_20_;
        t_124_ = t_124_*inv_20_;
        t_125_ = t_125_*inv_20_;
        t_126_ = t_126_*inv_20_;
        t_127_ = t_127_*inv_20_;
        t_128_ = t_92_*t_26_-t_25_*t_93_;
        t_129_ = t_92_*t_27_;
        t_130_ =  -(t_25_*t_94_);
        t_131_ = t_92_*t_28_-t_25_*t_95_;
        inv_21_ = safeinv(max(max(max(abs(t_128_), abs(t_129_)), abs(t_130_)), abs(t_131_)));
        t_128_ = t_128_*inv_21_;
        t_129_ = t_129_*inv_21_;
        t_130_ = t_130_*inv_21_;
        t_131_ = t_131_*inv_21_;
        t_132_ = t_92_*t_29_;
        t_133_ = t_92_*t_31_-t_30_*t_93_;
        t_134_ =  -(t_30_*t_94_);
        t_135_ = t_92_*t_32_-t_30_*t_95_;
        inv_22_ = safeinv(max(max(max(abs(t_132_), abs(t_133_)), abs(t_134_)), abs(t_135_)));
        t_132_ = t_132_*inv_22_;
        t_133_ = t_133_*inv_22_;
        t_134_ = t_134_*inv_22_;
        t_135_ = t_135_*inv_22_;
        t_136_ = t_92_*t_72_-t_71_*t_93_;
        t_137_ = t_92_*t_73_;
        t_138_ = t_92_*t_74_-t_71_*t_94_;
        t_139_ = t_92_*t_75_-t_71_*t_95_;
        inv_23_ = safeinv(max(max(max(abs(t_136_), abs(t_137_)), abs(t_138_)), abs(t_139_)));
        t_136_ = t_136_*inv_23_;
        t_137_ = t_137_*inv_23_;
        t_138_ = t_138_*inv_23_;
        t_139_ = t_139_*inv_23_;
        t_140_ = t_92_*t_76_;
        t_141_ = t_92_*t_78_-t_77_*t_93_;
        t_142_ = t_92_*t_79_;
        t_143_ = t_92_*t_80_-t_77_*t_94_;
        t_144_ = t_92_*t_81_;
        t_145_ = t_92_*t_82_-t_77_*t_95_;
        inv_24_ = safeinv(max(max(max(max(max(abs(t_140_), abs(t_141_)), abs(t_142_)), abs(t_143_)), abs(t_144_)), abs(t_145_)));
        t_140_ = t_140_*inv_24_;
        t_141_ = t_141_*inv_24_;
        t_142_ = t_142_*inv_24_;
        t_143_ = t_143_*inv_24_;
        t_144_ = t_144_*inv_24_;
        t_145_ = t_145_*inv_24_;
        t_146_ = t_92_*t_107_;
        t_147_ = t_92_*t_109_-t_108_*t_93_;
        t_148_ = t_92_*t_110_;
        t_149_ = t_92_*t_111_-t_108_*t_94_;
        t_150_ = t_92_*t_112_;
        t_151_ = t_92_*t_113_;
        t_152_ = t_92_*t_114_-t_108_*t_95_;
        inv_25_ = safeinv(max(max(max(max(max(max(abs(t_146_), abs(t_147_)), abs(t_148_)), abs(t_149_)), abs(t_150_)), abs(t_151_)), abs(t_152_)));
        t_146_ = t_146_*inv_25_;
        t_147_ = t_147_*inv_25_;
        t_148_ = t_148_*inv_25_;
        t_149_ = t_149_*inv_25_;
        t_150_ = t_150_*inv_25_;
        t_151_ = t_151_*inv_25_;
        t_152_ = t_152_*inv_25_;
        t_153_ = l_24_*t_124_;
        t_154_ = l_24_*t_125_;
        t_155_ = l_24_*t_126_;
        t_156_ =  -(t_123_*l_25_);
        t_157_ =  -(t_123_*l_26_);
        t_158_ = l_24_*t_127_-t_123_*l_27_;
        inv_26_ = safeinv(max(max(max(max(max(abs(t_153_), abs(t_154_)), abs(t_155_)), abs(t_156_)), abs(t_157_)), abs(t_158_)));
        t_153_ = t_153_*inv_26_;
        t_154_ = t_154_*inv_26_;
        t_155_ = t_155_*inv_26_;
        t_156_ = t_156_*inv_26_;
        t_157_ = t_157_*inv_26_;
        t_158_ = t_158_*inv_26_;
        t_159_ = l_24_*t_45_;
        t_160_ = l_24_*t_46_;
        t_161_ = l_24_*t_47_-t_44_*l_25_;
        t_162_ = l_24_*t_48_-t_44_*l_26_;
        t_163_ = l_24_*t_49_-t_44_*l_27_;
        inv_27_ = safeinv(max(max(max(max(abs(t_159_), abs(t_160_)), abs(t_161_)), abs(t_162_)), abs(t_163_)));
        t_159_ = t_159_*inv_27_;
        t_160_ = t_160_*inv_27_;
        t_161_ = t_161_*inv_27_;
        t_162_ = t_162_*inv_27_;
        t_163_ = t_163_*inv_27_;
        t_164_ = l_24_*t_141_;
        t_165_ = l_24_*t_142_;
        t_166_ = l_24_*t_143_;
        t_167_ =  -(t_140_*l_25_);
        t_168_ = l_24_*t_144_-t_140_*l_26_;
        t_169_ = l_24_*t_145_-t_140_*l_27_;
        inv_28_ = safeinv(max(max(max(max(max(abs(t_164_), abs(t_165_)), abs(t_166_)), abs(t_167_)), abs(t_168_)), abs(t_169_)));
        t_164_ = t_164_*inv_28_;
        t_165_ = t_165_*inv_28_;
        t_166_ = t_166_*inv_28_;
        t_167_ = t_167_*inv_28_;
        t_168_ = t_168_*inv_28_;
        t_169_ = t_169_*inv_28_;
        t_170_ = l_24_*l_47_;
        t_171_ = l_24_*l_49_;
        t_172_ =  -(l_48_*l_25_);
        t_173_ = l_24_*l_50_-l_48_*l_26_;
        t_174_ = l_24_*l_51_-l_48_*l_27_;
        inv_29_ = safeinv(max(max(max(max(abs(t_170_), abs(t_171_)), abs(t_172_)), abs(t_173_)), abs(t_174_)));
        t_170_ = t_170_*inv_29_;
        t_171_ = t_171_*inv_29_;
        t_172_ = t_172_*inv_29_;
        t_173_ = t_173_*inv_29_;
        t_174_ = t_174_*inv_29_;
        t_175_ = l_24_*t_147_;
        t_176_ = l_24_*t_148_;
        t_177_ = l_24_*t_149_;
        t_178_ = l_24_*t_150_-t_146_*l_25_;
        t_179_ = l_24_*t_151_-t_146_*l_26_;
        t_180_ = l_24_*t_152_-t_146_*l_27_;
        inv_30_ = safeinv(max(max(max(max(max(abs(t_175_), abs(t_176_)), abs(t_177_)), abs(t_178_)), abs(t_179_)), abs(t_180_)));
        t_175_ = t_175_*inv_30_;
        t_176_ = t_176_*inv_30_;
        t_177_ = t_177_*inv_30_;
        t_178_ = t_178_*inv_30_;
        t_179_ = t_179_*inv_30_;
        t_180_ = t_180_*inv_30_;
        t_181_ = t_159_*t_119_;
        t_182_ = t_159_*t_121_-t_120_*t_160_;
        t_183_ =  -(t_120_*t_161_);
        t_184_ =  -(t_120_*t_162_);
        t_185_ = t_159_*t_122_-t_120_*t_163_;
        inv_31_ = safeinv(max(max(max(max(abs(t_181_), abs(t_182_)), abs(t_183_)), abs(t_184_)), abs(t_185_)));
        t_181_ = t_181_*inv_31_;
        t_182_ = t_182_*inv_31_;
        t_183_ = t_183_*inv_31_;
        t_184_ = t_184_*inv_31_;
        t_185_ = t_185_*inv_31_;
        t_186_ = t_159_*t_153_;
        t_187_ = t_159_*t_155_-t_154_*t_160_;
        t_188_ = t_159_*t_156_-t_154_*t_161_;
        t_189_ = t_159_*t_157_-t_154_*t_162_;
        t_190_ = t_159_*t_158_-t_154_*t_163_;
        inv_32_ = safeinv(max(max(max(max(abs(t_186_), abs(t_187_)), abs(t_188_)), abs(t_189_)), abs(t_190_)));
        t_186_ = t_186_*inv_32_;
        t_187_ = t_187_*inv_32_;
        t_188_ = t_188_*inv_32_;
        t_189_ = t_189_*inv_32_;
        t_190_ = t_190_*inv_32_;
        t_191_ = t_159_*t_164_;
        t_192_ = t_159_*t_166_-t_165_*t_160_;
        t_193_ = t_159_*t_167_-t_165_*t_161_;
        t_194_ = t_159_*t_168_-t_165_*t_162_;
        t_195_ = t_159_*t_169_-t_165_*t_163_;
        inv_33_ = safeinv(max(max(max(max(abs(t_191_), abs(t_192_)), abs(t_193_)), abs(t_194_)), abs(t_195_)));
        t_191_ = t_191_*inv_33_;
        t_192_ = t_192_*inv_33_;
        t_193_ = t_193_*inv_33_;
        t_194_ = t_194_*inv_33_;
        t_195_ = t_195_*inv_33_;
        t_196_ = t_159_*t_175_;
        t_197_ = t_159_*t_177_-t_176_*t_160_;
        t_198_ = t_159_*t_178_-t_176_*t_161_;
        t_199_ = t_159_*t_179_-t_176_*t_162_;
        t_200_ = t_159_*t_180_-t_176_*t_163_;
        inv_34_ = safeinv(max(max(max(max(abs(t_196_), abs(t_197_)), abs(t_198_)), abs(t_199_)), abs(t_200_)));
        t_196_ = t_196_*inv_34_;
        t_197_ = t_197_*inv_34_;
        t_198_ = t_198_*inv_34_;
        t_199_ = t_199_*inv_34_;
        t_200_ = t_200_*inv_34_;
        t_201_ = t_199_*t_181_-t_184_*t_196_;
        t_202_ = t_199_*t_182_-t_184_*t_197_;
        t_203_ = t_199_*t_183_-t_184_*t_198_;
        t_204_ = t_199_*t_185_-t_184_*t_200_;
        inv_35_ = safeinv(max(max(max(abs(t_201_), abs(t_202_)), abs(t_203_)), abs(t_204_)));
        t_201_ = t_201_*inv_35_;
        t_202_ = t_202_*inv_35_;
        t_203_ = t_203_*inv_35_;
        t_204_ = t_204_*inv_35_;
        t_205_ = t_199_*t_186_-t_189_*t_196_;
        t_206_ = t_199_*t_187_-t_189_*t_197_;
        t_207_ = t_199_*t_188_-t_189_*t_198_;
        t_208_ = t_199_*t_190_-t_189_*t_200_;
        inv_36_ = safeinv(max(max(max(abs(t_205_), abs(t_206_)), abs(t_207_)), abs(t_208_)));
        t_205_ = t_205_*inv_36_;
        t_206_ = t_206_*inv_36_;
        t_207_ = t_207_*inv_36_;
        t_208_ = t_208_*inv_36_;
        t_209_ =  -(t_162_*t_196_);
        t_210_ = t_199_*t_159_;
        t_211_ = t_199_*t_160_-t_162_*t_197_;
        t_212_ = t_199_*t_161_-t_162_*t_198_;
        t_213_ = t_199_*t_163_-t_162_*t_200_;
        inv_37_ = safeinv(max(max(max(max(abs(t_209_), abs(t_210_)), abs(t_211_)), abs(t_212_)), abs(t_213_)));
        t_209_ = t_209_*inv_37_;
        t_210_ = t_210_*inv_37_;
        t_211_ = t_211_*inv_37_;
        t_212_ = t_212_*inv_37_;
        t_213_ = t_213_*inv_37_;
        t_214_ = t_199_*l_24_;
        t_215_ =  -(l_26_*t_196_);
        t_216_ =  -(l_26_*t_197_);
        t_217_ = t_199_*l_25_-l_26_*t_198_;
        t_218_ = t_199_*l_27_-l_26_*t_200_;
        inv_38_ = safeinv(max(max(max(max(abs(t_214_), abs(t_215_)), abs(t_216_)), abs(t_217_)), abs(t_218_)));
        t_214_ = t_214_*inv_38_;
        t_215_ = t_215_*inv_38_;
        t_216_ = t_216_*inv_38_;
        t_217_ = t_217_*inv_38_;
        t_218_ = t_218_*inv_38_;
        t_219_ = t_199_*t_191_-t_194_*t_196_;
        t_220_ = t_199_*t_192_-t_194_*t_197_;
        t_221_ = t_199_*t_193_-t_194_*t_198_;
        t_222_ = t_199_*t_195_-t_194_*t_200_;
        inv_39_ = safeinv(max(max(max(abs(t_219_), abs(t_220_)), abs(t_221_)), abs(t_222_)));
        t_219_ = t_219_*inv_39_;
        t_220_ = t_220_*inv_39_;
        t_221_ = t_221_*inv_39_;
        t_222_ = t_222_*inv_39_;
        t_223_ = t_199_*t_170_;
        t_224_ =  -(t_173_*t_196_);
        t_225_ = t_199_*t_171_-t_173_*t_197_;
        t_226_ = t_199_*t_172_-t_173_*t_198_;
        t_227_ = t_199_*t_174_-t_173_*t_200_;
        inv_40_ = safeinv(max(max(max(max(abs(t_223_), abs(t_224_)), abs(t_225_)), abs(t_226_)), abs(t_227_)));
        t_223_ = t_223_*inv_40_;
        t_224_ = t_224_*inv_40_;
        t_225_ = t_225_*inv_40_;
        t_226_ = t_226_*inv_40_;
        t_227_ = t_227_*inv_40_;
        t_228_ = t_220_*t_115_;
        t_229_ = t_220_*t_116_-t_117_*t_219_;
        t_230_ =  -(t_117_*t_221_);
        t_231_ = t_220_*t_118_-t_117_*t_222_;
        inv_41_ = safeinv(max(max(max(abs(t_228_), abs(t_229_)), abs(t_230_)), abs(t_231_)));
        t_228_ = t_228_*inv_41_;
        t_229_ = t_229_*inv_41_;
        t_230_ = t_230_*inv_41_;
        t_231_ = t_231_*inv_41_;
        t_232_ = t_220_*t_92_;
        t_233_ = t_220_*t_93_-t_94_*t_219_;
        t_234_ =  -(t_94_*t_221_);
        t_235_ = t_220_*t_95_-t_94_*t_222_;
        inv_42_ = safeinv(max(max(max(abs(t_232_), abs(t_233_)), abs(t_234_)), abs(t_235_)));
        t_232_ = t_232_*inv_42_;
        t_233_ = t_233_*inv_42_;
        t_234_ = t_234_*inv_42_;
        t_235_ = t_235_*inv_42_;
        t_236_ = t_220_*t_201_-t_202_*t_219_;
        t_237_ = t_220_*t_203_-t_202_*t_221_;
        t_238_ = t_220_*t_204_-t_202_*t_222_;
        inv_43_ = safeinv(max(max(abs(t_236_), abs(t_237_)), abs(t_238_)));
        t_236_ = t_236_*inv_43_;
        t_237_ = t_237_*inv_43_;
        t_238_ = t_238_*inv_43_;
        t_239_ = t_220_*t_205_-t_206_*t_219_;
        t_240_ = t_220_*t_207_-t_206_*t_221_;
        t_241_ = t_220_*t_208_-t_206_*t_222_;
        inv_44_ = safeinv(max(max(abs(t_239_), abs(t_240_)), abs(t_241_)));
        t_239_ = t_239_*inv_44_;
        t_240_ = t_240_*inv_44_;
        t_241_ = t_241_*inv_44_;
        t_242_ = t_220_*t_209_-t_211_*t_219_;
        t_243_ = t_220_*t_210_;
        t_244_ = t_220_*t_212_-t_211_*t_221_;
        t_245_ = t_220_*t_213_-t_211_*t_222_;
        inv_45_ = safeinv(max(max(max(abs(t_242_), abs(t_243_)), abs(t_244_)), abs(t_245_)));
        t_242_ = t_242_*inv_45_;
        t_243_ = t_243_*inv_45_;
        t_244_ = t_244_*inv_45_;
        t_245_ = t_245_*inv_45_;
        t_246_ = t_220_*t_214_;
        t_247_ = t_220_*t_215_-t_216_*t_219_;
        t_248_ = t_220_*t_217_-t_216_*t_221_;
        t_249_ = t_220_*t_218_-t_216_*t_222_;
        inv_46_ = safeinv(max(max(max(abs(t_246_), abs(t_247_)), abs(t_248_)), abs(t_249_)));
        t_246_ = t_246_*inv_46_;
        t_247_ = t_247_*inv_46_;
        t_248_ = t_248_*inv_46_;
        t_249_ = t_249_*inv_46_;
        t_250_ = t_220_*t_128_-t_130_*t_219_;
        t_251_ = t_220_*t_129_;
        t_252_ =  -(t_130_*t_221_);
        t_253_ = t_220_*t_131_-t_130_*t_222_;
        inv_47_ = safeinv(max(max(max(abs(t_250_), abs(t_251_)), abs(t_252_)), abs(t_253_)));
        t_250_ = t_250_*inv_47_;
        t_251_ = t_251_*inv_47_;
        t_252_ = t_252_*inv_47_;
        t_253_ = t_253_*inv_47_;
        t_254_ = t_220_*t_132_;
        t_255_ = t_220_*t_133_-t_134_*t_219_;
        t_256_ =  -(t_134_*t_221_);
        t_257_ = t_220_*t_135_-t_134_*t_222_;
        inv_48_ = safeinv(max(max(max(abs(t_254_), abs(t_255_)), abs(t_256_)), abs(t_257_)));
        t_254_ = t_254_*inv_48_;
        t_255_ = t_255_*inv_48_;
        t_256_ = t_256_*inv_48_;
        t_257_ = t_257_*inv_48_;
        t_258_ = t_220_*t_136_-t_138_*t_219_;
        t_259_ = t_220_*t_137_;
        t_260_ =  -(t_138_*t_221_);
        t_261_ = t_220_*t_139_-t_138_*t_222_;
        inv_49_ = safeinv(max(max(max(abs(t_258_), abs(t_259_)), abs(t_260_)), abs(t_261_)));
        t_258_ = t_258_*inv_49_;
        t_259_ = t_259_*inv_49_;
        t_260_ = t_260_*inv_49_;
        t_261_ = t_261_*inv_49_;
        t_262_ = t_220_*t_223_;
        t_263_ = t_220_*t_224_-t_225_*t_219_;
        t_264_ = t_220_*t_226_-t_225_*t_221_;
        t_265_ = t_220_*t_227_-t_225_*t_222_;
        inv_50_ = safeinv(max(max(max(abs(t_262_), abs(t_263_)), abs(t_264_)), abs(t_265_)));
        t_262_ = t_262_*inv_50_;
        t_263_ = t_263_*inv_50_;
        t_264_ = t_264_*inv_50_;
        t_265_ = t_265_*inv_50_;
        t_266_ = t_220_*t_196_-t_197_*t_219_;
        t_267_ = t_220_*t_198_-t_197_*t_221_;
        t_268_ = t_220_*t_199_;
        t_269_ = t_220_*t_200_-t_197_*t_222_;
        inv_51_ = safeinv(max(max(max(abs(t_266_), abs(t_267_)), abs(t_268_)), abs(t_269_)));
        t_266_ = t_266_*inv_51_;
        t_267_ = t_267_*inv_51_;
        t_268_ = t_268_*inv_51_;
        t_269_ = t_269_*inv_51_;
        t_270_ = t_239_*t_228_;
        t_271_ = t_239_*t_230_-t_229_*t_240_;
        t_272_ = t_239_*t_231_-t_229_*t_241_;
        inv_52_ = safeinv(max(max(abs(t_270_), abs(t_271_)), abs(t_272_)));
        t_270_ = t_270_*inv_52_;
        t_271_ = t_271_*inv_52_;
        t_272_ = t_272_*inv_52_;
        t_273_ = t_239_*t_232_;
        t_274_ = t_239_*t_234_-t_233_*t_240_;
        t_275_ = t_239_*t_235_-t_233_*t_241_;
        inv_53_ = safeinv(max(max(abs(t_273_), abs(t_274_)), abs(t_275_)));
        t_273_ = t_273_*inv_53_;
        t_274_ = t_274_*inv_53_;
        t_275_ = t_275_*inv_53_;
        t_276_ = t_239_*t_237_-t_236_*t_240_;
        t_277_ = t_239_*t_238_-t_236_*t_241_;
        inv_54_ = safeinv(max(abs(t_276_), abs(t_277_)));
        t_276_ = t_276_*inv_54_;
        t_277_ = t_277_*inv_54_;
        t_278_ = t_239_*t_243_;
        t_279_ = t_239_*t_244_-t_242_*t_240_;
        t_280_ = t_239_*t_245_-t_242_*t_241_;
        inv_55_ = safeinv(max(max(abs(t_278_), abs(t_279_)), abs(t_280_)));
        t_278_ = t_278_*inv_55_;
        t_279_ = t_279_*inv_55_;
        t_280_ = t_280_*inv_55_;
        t_281_ = t_239_*t_246_;
        t_282_ = t_239_*t_248_-t_247_*t_240_;
        t_283_ = t_239_*t_249_-t_247_*t_241_;
        inv_56_ = safeinv(max(max(abs(t_281_), abs(t_282_)), abs(t_283_)));
        t_281_ = t_281_*inv_56_;
        t_282_ = t_282_*inv_56_;
        t_283_ = t_283_*inv_56_;
        t_284_ = t_239_*t_251_;
        t_285_ = t_239_*t_252_-t_250_*t_240_;
        t_286_ = t_239_*t_253_-t_250_*t_241_;
        inv_57_ = safeinv(max(max(abs(t_284_), abs(t_285_)), abs(t_286_)));
        t_284_ = t_284_*inv_57_;
        t_285_ = t_285_*inv_57_;
        t_286_ = t_286_*inv_57_;
        t_287_ = t_239_*t_254_;
        t_288_ = t_239_*t_256_-t_255_*t_240_;
        t_289_ = t_239_*t_257_-t_255_*t_241_;
        inv_58_ = safeinv(max(max(abs(t_287_), abs(t_288_)), abs(t_289_)));
        t_287_ = t_287_*inv_58_;
        t_288_ = t_288_*inv_58_;
        t_289_ = t_289_*inv_58_;
        t_290_ = t_239_*t_259_;
        t_291_ = t_239_*t_260_-t_258_*t_240_;
        t_292_ = t_239_*t_261_-t_258_*t_241_;
        inv_59_ = safeinv(max(max(abs(t_290_), abs(t_291_)), abs(t_292_)));
        t_290_ = t_290_*inv_59_;
        t_291_ = t_291_*inv_59_;
        t_292_ = t_292_*inv_59_;
        t_293_ = t_239_*t_220_;
        t_294_ = t_239_*t_221_-t_219_*t_240_;
        t_295_ = t_239_*t_222_-t_219_*t_241_;
        inv_60_ = safeinv(max(max(abs(t_293_), abs(t_294_)), abs(t_295_)));
        t_293_ = t_293_*inv_60_;
        t_294_ = t_294_*inv_60_;
        t_295_ = t_295_*inv_60_;
        t_296_ = t_239_*t_262_;
        t_297_ = t_239_*t_264_-t_263_*t_240_;
        t_298_ = t_239_*t_265_-t_263_*t_241_;
        inv_61_ = safeinv(max(max(abs(t_296_), abs(t_297_)), abs(t_298_)));
        t_296_ = t_296_*inv_61_;
        t_297_ = t_297_*inv_61_;
        t_298_ = t_298_*inv_61_;
        t_299_ = t_239_*t_267_-t_266_*t_240_;
        t_300_ = t_239_*t_268_;
        t_301_ = t_239_*t_269_-t_266_*t_241_;
        inv_62_ = safeinv(max(max(abs(t_299_), abs(t_300_)), abs(t_301_)));
        t_299_ = t_299_*inv_62_;
        t_300_ = t_300_*inv_62_;
        t_301_ = t_301_*inv_62_;
        t_302_ = t_276_*t_270_;
        t_303_ = t_276_*t_272_-t_271_*t_277_;
        inv_63_ = safeinv(max(abs(t_302_), abs(t_303_)));
        t_302_ = t_302_*inv_63_;
        t_303_ = t_303_*inv_63_;
        t_304_ = t_276_*t_273_;
        t_305_ = t_276_*t_275_-t_274_*t_277_;
        inv_64_ = safeinv(max(abs(t_304_), abs(t_305_)));
        t_304_ = t_304_*inv_64_;
        t_305_ = t_305_*inv_64_;
        t_306_ = t_276_*t_239_;
        t_307_ = t_276_*t_241_-t_240_*t_277_;
        inv_65_ = safeinv(max(abs(t_306_), abs(t_307_)));
        t_306_ = t_306_*inv_65_;
        t_307_ = t_307_*inv_65_;
        t_308_ = t_276_*t_278_;
        t_309_ = t_276_*t_280_-t_279_*t_277_;
        inv_66_ = safeinv(max(abs(t_308_), abs(t_309_)));
        t_308_ = t_308_*inv_66_;
        t_309_ = t_309_*inv_66_;
        t_310_ = t_276_*t_281_;
        t_311_ = t_276_*t_283_-t_282_*t_277_;
        inv_67_ = safeinv(max(abs(t_310_), abs(t_311_)));
        t_310_ = t_310_*inv_67_;
        t_311_ = t_311_*inv_67_;
        t_312_ = t_276_*t_284_;
        t_313_ = t_276_*t_286_-t_285_*t_277_;
        inv_68_ = safeinv(max(abs(t_312_), abs(t_313_)));
        t_312_ = t_312_*inv_68_;
        t_313_ = t_313_*inv_68_;
        t_314_ = t_276_*t_287_;
        t_315_ = t_276_*t_289_-t_288_*t_277_;
        inv_69_ = safeinv(max(abs(t_314_), abs(t_315_)));
        t_314_ = t_314_*inv_69_;
        t_315_ = t_315_*inv_69_;
        t_316_ = t_276_*t_290_;
        t_317_ = t_276_*t_292_-t_291_*t_277_;
        inv_70_ = safeinv(max(abs(t_316_), abs(t_317_)));
        t_316_ = t_316_*inv_70_;
        t_317_ = t_317_*inv_70_;
        t_318_ = t_276_*t_293_;
        t_319_ = t_276_*t_295_-t_294_*t_277_;
        inv_71_ = safeinv(max(abs(t_318_), abs(t_319_)));
        t_318_ = t_318_*inv_71_;
        t_319_ = t_319_*inv_71_;
        t_320_ = t_276_*t_296_;
        t_321_ = t_276_*t_298_-t_297_*t_277_;
        inv_72_ = safeinv(max(abs(t_320_), abs(t_321_)));
        t_320_ = t_320_*inv_72_;
        t_321_ = t_321_*inv_72_;
        t_322_ = t_276_*t_300_;
        t_323_ = t_276_*t_301_-t_299_*t_277_;
        inv_73_ = safeinv(max(abs(t_322_), abs(t_323_)));
        t_322_ = t_322_*inv_73_;
        t_323_ = t_323_*inv_73_;
        _pp_var_C1[i_] = t_303_/t_302_;
        _pp_var_I2[i_] = t_305_/t_304_;
        _pp_var_O[i_] = t_277_/t_276_;
        _pp_var_C2[i_] = t_307_/t_306_;
        _pp_var_C4[i_] = t_309_/t_308_;
        _pp_var_C5[i_] = t_311_/t_310_;
        _pp_var_I1[i_] = t_313_/t_312_;
        _pp_var_I3[i_] = t_315_/t_314_;
        _pp_var_C3[i_] = t_317_/t_316_;
        _pp_var_I4[i_] = t_319_/t_318_;
        _pp_var_I5[i_] = t_321_/t_320_;
        _pp_var_I6[i_] = t_323_/t_322_;
    }
    if (!_pp_var_multiplicity) return;
    for (arb_size_type ix = 0; ix < 12; ++ix) {
        for (arb_size_type iy = 0; iy < _pp_var_width; ++iy) {
            pp->state_vars[ix][iy] *= _pp_var_multiplicity[iy];
        }
    }
}

static void advance_state(arb_mechanism_ppack* pp) {
    PPACK_IFACE_BLOCK;
    for (arb_size_type i_ = 0; i_ < _pp_var_width; ++i_) {
        auto node_indexi_ = _pp_var_node_index[i_];
        arb_value_type v = _pp_var_vec_v[node_indexi_];
        arb_value_type t_275_, inv_64_, t_274_, t_273_, inv_63_, t_269_, inv_61_, t_264_, t_263_, t_261_, t_260_, t_259_, t_257_, t_256_, t_255_, inv_54_, t_254_, t_253_, t_250_, t_249_, t_247_, inv_51_, t_245_, inv_50_, t_240_, inv_49_, t_239_, t_238_, t_237_, inv_48_, t_236_, inv_47_, t_233_, t_232_, t_231_, t_246_, inv_46_, t_229_, t_228_, t_271_, t_226_, inv_44_, t_224_, inv_43_, t_221_, t_220_, inv_42_, t_225_, t_219_, inv_62_, t_217_, t_216_, inv_41_, t_215_, t_214_, t_213_, inv_40_, t_211_, t_210_, t_209_, t_208_, t_205_, t_203_, t_268_, t_202_, t_201_, inv_37_, t_200_, t_199_, t_198_, t_197_, t_193_, t_192_, t_191_, t_190_, t_188_, t_258_, t_187_, t_186_, inv_33_, t_181_, t_180_, t_178_, inv_55_, t_175_, t_172_, t_204_, inv_30_, t_170_, t_166_, t_165_, t_162_, t_160_, t_159_, t_158_, t_168_, inv_27_, t_157_, inv_52_, t_155_, t_154_, inv_31_, inv_26_, t_234_, t_152_, inv_60_, t_151_, t_177_, t_149_, t_148_, t_265_, inv_25_, t_147_, b14, a_37_, t_251_, t_176_, t_56_, t_30_, inv_2_, t_59_, t_25_, t_36_, t_167_, t_24_, t_21_, t_270_, t_19_, inv_6_, t_102_, t_242_, t_18_, a_1_, inv_59_, inv_1_, t_13_, bi2, t_12_, t_31_, t_8_, a_43_, inv_24_, a_20_, bin, t_150_, t_0_, a_52_, t_55_, t_113_, t_243_, t_32_, t_131_, a_50_, t_194_, a_0_, a_49_, a_46_, a_48_, a_45_, a_44_, a_30_, a_42_, t_207_, t_7_, f02, t_223_, t_9_, t_206_, t_51_, f13, inv_35_, a_34_, t_67_, inv_32_, t_92_, t_143_, a_33_, a_31_, a_26_, t_35_, inv_17_, a_28_, inv_65_, t_272_, a_27_, a_17_, t_173_, a_39_, t_179_, t_52_, fi3, t_184_, t_146_, t_127_, b13, fin, b01, a_47_, t_222_, t_116_, t_262_, b0O, t_99_, t_128_, t_3_, f04, inv_29_, a_41_, bi1, t_22_, a_19_, t_10_, t_80_, b11, f11, bi5, inv_58_, t_41_, t_82_, t_38_, t_125_, b02, inv_45_, t_212_, t_73_, f03, a_9_, inv_28_, t_144_, t_124_, f01, t_11_, inv_56_, fi2, inv_8_, a_53_, bi3, t_227_, t_91_, a_18_, fi5, a_38_, t_84_, b04, t_183_, a_23_, t_4_, fi4, t_1_, t_14_, inv_10_, t_132_, inv_53_, f14, a_10_, t_126_, t_145_, ibtf, inv_57_, fi1, t_218_, inv_36_, t_161_, t_16_, t_15_, t_66_, bi4, t_118_, inv_39_, t_61_, a_14_, inv_34_, a_12_, a_15_, t_75_, t_86_, t_72_, a_3_, f12, inv_22_, a_4_, a_32_, a_6_, t_156_, t_28_, a_7_, inv_38_, b12, t_140_, a_24_, a_16_, inv_3_, t_23_, a_21_, t_26_, a_13_, a_11_, t_235_, t_182_, a_22_, t_29_, t_5_, t_142_, a_25_, t_6_, inv_4_, t_123_, t_34_, t_37_, t_33_, t_50_, t_44_, t_171_, t_48_, t_17_, t_45_, t_46_, inv_5_, t_71_, f0O, inv_7_, t_266_, t_53_, t_103_, t_109_, t_54_, t_58_, t_133_, t_63_, a_29_, t_64_, t_65_, t_196_, inv_9_, t_68_, inv_0_, t_95_, t_57_, inv_14_, b03, inv_16_, t_69_, t_79_, t_70_, t_153_, t_74_, t_100_, t_76_, t_195_, t_141_, t_47_, t_77_, t_78_, t_110_, t_139_, t_163_, t_2_, inv_11_, t_83_, a_40_, t_49_, t_85_, a_35_, t_62_, t_89_, inv_12_, t_87_, t_60_, inv_19_, t_88_, t_90_, t_276_, t_244_, a_8_, inv_13_, t_267_, t_93_, t_252_, t_94_, t_96_, t_138_, t_97_, t_81_, t_98_, t_230_, a_51_, inv_15_, t_27_, t_101_, t_104_, t_43_, a_5_, t_105_, t_169_, inv_18_, t_106_, t_107_, a_36_, t_111_, t_241_, t_112_, t_174_, t_39_, t_114_, t_115_, t_117_, t_185_, t_20_, t_119_, t_120_, t_121_, t_122_, t_40_, inv_20_, inv_21_, t_189_, t_129_, t_248_, a_2_, t_130_, t_134_, t_164_, t_135_, t_136_, t_108_, inv_23_, t_42_, t_137_;
        ibtf =  1.0/_pp_var_btfac;
        f04 = _pp_var_qt[i_]*_pp_var_alpha*exp(v/_pp_var_x1);
        f03 =  2.0*f04;
        f02 =  3.0*f04;
        f01 =  4.0*f04;
        f0O = _pp_var_qt[i_]*_pp_var_gamma;
        f14 = _pp_var_alfac*f04;
        f13 =  2.0*f14;
        f12 =  3.0*f14;
        f11 =  4.0*f14;
        fi1 = _pp_var_qt[i_]*_pp_var_Con;
        fi2 = fi1*_pp_var_alfac;
        fi3 = fi2*_pp_var_alfac;
        fi4 = fi3*_pp_var_alfac;
        fi5 = fi4*_pp_var_alfac;
        fin = _pp_var_qt[i_]*_pp_var_Oon;
        b01 = _pp_var_qt[i_]*_pp_var_beta*exp(v/_pp_var_x2);
        b02 =  2.0*b01;
        b03 =  3.0*b01;
        b04 =  4.0*b01;
        b0O = _pp_var_qt[i_]*_pp_var_delta;
        b11 = b01*ibtf;
        b12 =  2.0*b11;
        b13 =  3.0*b11;
        b14 =  4.0*b11;
        bi1 = _pp_var_qt[i_]*_pp_var_Coff;
        bi2 = bi1*ibtf;
        bi3 = bi2*ibtf;
        bi4 = bi3*ibtf;
        bi5 = bi4*ibtf;
        bin = _pp_var_qt[i_]*_pp_var_Ooff;
        a_0_ =  1.0;
        a_1_ =  1.0;
        a_2_ =  1.0;
        a_3_ =  1.0;
        a_4_ =  1.0;
        a_5_ =  1.0;
        a_6_ =  1.0;
        a_7_ =  1.0;
        a_8_ =  1.0;
        a_9_ =  1.0;
        a_10_ =  1.0;
        a_11_ =  1.0;
        a_12_ =  1.0;
        a_13_ =  -(f01*_pp_var_dt);
        a_14_ =  1.0-( -b01+ -1.0*f02+ -1.0*fi2)*_pp_var_dt;
        a_15_ =  -( -1.0* -b02*_pp_var_dt);
        a_16_ =  -( -1.0* -bi2*_pp_var_dt);
        a_17_ =  -(f02*_pp_var_dt);
        a_18_ =  1.0-( -b02+ -1.0*f03+ -1.0*fi3)*_pp_var_dt;
        a_19_ =  -( -1.0* -b03*_pp_var_dt);
        a_20_ =  -( -1.0* -bi3*_pp_var_dt);
        a_21_ =  -(f03*_pp_var_dt);
        a_22_ =  1.0-( -b03+ -1.0*f04+ -1.0*fi4)*_pp_var_dt;
        a_23_ =  -( -1.0* -b04*_pp_var_dt);
        a_24_ =  -( -1.0* -bi4*_pp_var_dt);
        a_25_ =  -(f04*_pp_var_dt);
        a_26_ =  1.0-( -b04+ -1.0*f0O+ -1.0*fi5)*_pp_var_dt;
        a_27_ =  -( -1.0* -bi5*_pp_var_dt);
        a_28_ =  -( -1.0* -b0O*_pp_var_dt);
        a_29_ =  -(fi1*_pp_var_dt);
        a_30_ =  1.0-( -1.0*f11+ -bi1)*_pp_var_dt;
        a_31_ =  -( -1.0* -b11*_pp_var_dt);
        a_32_ =  -(fi2*_pp_var_dt);
        a_33_ =  -(f11*_pp_var_dt);
        a_34_ =  1.0-( -b11+ -1.0*f12+ -bi2)*_pp_var_dt;
        a_35_ =  -( -1.0* -b12*_pp_var_dt);
        a_36_ =  -(fi3*_pp_var_dt);
        a_37_ =  -(f12*_pp_var_dt);
        a_38_ =  1.0-( -b12+ -1.0*f13+ -bi3)*_pp_var_dt;
        a_39_ =  -( -1.0* -b13*_pp_var_dt);
        a_40_ =  -(fi4*_pp_var_dt);
        a_41_ =  -(f13*_pp_var_dt);
        a_42_ =  1.0-( -b13+ -1.0*f14+ -bi4)*_pp_var_dt;
        a_43_ =  -( -1.0* -b14*_pp_var_dt);
        a_44_ =  -(fi5*_pp_var_dt);
        a_45_ =  -(f14*_pp_var_dt);
        a_46_ =  1.0-( -b14+ -1.0*f0O+ -bi5)*_pp_var_dt;
        a_47_ =  -( -1.0* -b0O*_pp_var_dt);
        a_48_ =  -(f0O*_pp_var_dt);
        a_49_ =  1.0-( -bin+ -b0O)*_pp_var_dt;
        a_50_ =  -(fin*_pp_var_dt);
        a_51_ =  -(f0O*_pp_var_dt);
        a_52_ =  -( -1.0* -bin*_pp_var_dt);
        a_53_ =  1.0-( -b0O+ -1.0*fin)*_pp_var_dt;
        t_0_ = a_30_*a_0_-a_5_*a_29_;
        t_1_ = a_30_*a_1_;
        t_2_ = a_30_*a_2_;
        t_3_ = a_30_*a_3_;
        t_4_ = a_30_*a_4_;
        t_5_ = a_30_*a_6_-a_5_*a_31_;
        t_6_ = a_30_*a_7_;
        t_7_ = a_30_*a_8_;
        t_8_ = a_30_*a_9_;
        t_9_ = a_30_*a_10_;
        t_10_ = a_30_*a_11_;
        t_11_ = a_30_*a_12_-a_5_*_pp_var_I1[i_];
        inv_0_ = safeinv(max(max(max(max(max(max(max(max(max(max(max(abs(t_0_), abs(t_1_)), abs(t_2_)), abs(t_3_)), abs(t_4_)), abs(t_5_)), abs(t_6_)), abs(t_7_)), abs(t_8_)), abs(t_9_)), abs(t_10_)), abs(t_11_)));
        t_0_ = t_0_*inv_0_;
        t_1_ = t_1_*inv_0_;
        t_2_ = t_2_*inv_0_;
        t_3_ = t_3_*inv_0_;
        t_4_ = t_4_*inv_0_;
        t_5_ = t_5_*inv_0_;
        t_6_ = t_6_*inv_0_;
        t_7_ = t_7_*inv_0_;
        t_8_ = t_8_*inv_0_;
        t_9_ = t_9_*inv_0_;
        t_10_ = t_10_*inv_0_;
        t_11_ = t_11_*inv_0_;
        t_12_ =  -(a_33_*a_29_);
        t_13_ = a_30_*a_32_;
        t_14_ = a_30_*a_34_-a_33_*a_31_;
        t_15_ = a_30_*a_35_;
        t_16_ = a_30_*_pp_var_I2[i_]-a_33_*_pp_var_I1[i_];
        inv_1_ = safeinv(max(max(max(max(abs(t_12_), abs(t_13_)), abs(t_14_)), abs(t_15_)), abs(t_16_)));
        t_12_ = t_12_*inv_1_;
        t_13_ = t_13_*inv_1_;
        t_14_ = t_14_*inv_1_;
        t_15_ = t_15_*inv_1_;
        t_16_ = t_16_*inv_1_;
        t_17_ = a_53_*t_0_;
        t_18_ = a_53_*t_1_;
        t_19_ = a_53_*t_2_;
        t_20_ = a_53_*t_3_;
        t_21_ = a_53_*t_4_-t_10_*a_51_;
        t_22_ = a_53_*t_5_;
        t_23_ = a_53_*t_6_;
        t_24_ = a_53_*t_7_;
        t_25_ = a_53_*t_8_;
        t_26_ = a_53_*t_9_-t_10_*a_52_;
        t_27_ = a_53_*t_11_-t_10_*_pp_var_O[i_];
        inv_2_ = safeinv(max(max(max(max(max(max(max(max(max(max(abs(t_17_), abs(t_18_)), abs(t_19_)), abs(t_20_)), abs(t_21_)), abs(t_22_)), abs(t_23_)), abs(t_24_)), abs(t_25_)), abs(t_26_)), abs(t_27_)));
        t_17_ = t_17_*inv_2_;
        t_18_ = t_18_*inv_2_;
        t_19_ = t_19_*inv_2_;
        t_20_ = t_20_*inv_2_;
        t_21_ = t_21_*inv_2_;
        t_22_ = t_22_*inv_2_;
        t_23_ = t_23_*inv_2_;
        t_24_ = t_24_*inv_2_;
        t_25_ = t_25_*inv_2_;
        t_26_ = t_26_*inv_2_;
        t_27_ = t_27_*inv_2_;
        t_28_ = a_53_*a_25_;
        t_29_ = a_53_*a_26_-a_28_*a_51_;
        t_30_ = a_53_*a_27_;
        t_31_ =  -(a_28_*a_52_);
        t_32_ = a_53_*_pp_var_C5[i_]-a_28_*_pp_var_O[i_];
        inv_3_ = safeinv(max(max(max(max(abs(t_28_), abs(t_29_)), abs(t_30_)), abs(t_31_)), abs(t_32_)));
        t_28_ = t_28_*inv_3_;
        t_29_ = t_29_*inv_3_;
        t_30_ = t_30_*inv_3_;
        t_31_ = t_31_*inv_3_;
        t_32_ = t_32_*inv_3_;
        t_33_ =  -(a_50_*a_51_);
        t_34_ = a_53_*a_48_;
        t_35_ = a_53_*a_49_-a_50_*a_52_;
        t_36_ = a_53_*_pp_var_I6[i_]-a_50_*_pp_var_O[i_];
        inv_4_ = safeinv(max(max(max(abs(t_33_), abs(t_34_)), abs(t_35_)), abs(t_36_)));
        t_33_ = t_33_*inv_4_;
        t_34_ = t_34_*inv_4_;
        t_35_ = t_35_*inv_4_;
        t_36_ = t_36_*inv_4_;
        t_37_ = t_35_*t_17_;
        t_38_ = t_35_*t_18_;
        t_39_ = t_35_*t_19_;
        t_40_ = t_35_*t_20_;
        t_41_ = t_35_*t_21_-t_26_*t_33_;
        t_42_ = t_35_*t_22_;
        t_43_ = t_35_*t_23_;
        t_44_ = t_35_*t_24_;
        t_45_ = t_35_*t_25_-t_26_*t_34_;
        t_46_ = t_35_*t_27_-t_26_*t_36_;
        inv_5_ = safeinv(max(max(max(max(max(max(max(max(max(abs(t_37_), abs(t_38_)), abs(t_39_)), abs(t_40_)), abs(t_41_)), abs(t_42_)), abs(t_43_)), abs(t_44_)), abs(t_45_)), abs(t_46_)));
        t_37_ = t_37_*inv_5_;
        t_38_ = t_38_*inv_5_;
        t_39_ = t_39_*inv_5_;
        t_40_ = t_40_*inv_5_;
        t_41_ = t_41_*inv_5_;
        t_42_ = t_42_*inv_5_;
        t_43_ = t_43_*inv_5_;
        t_44_ = t_44_*inv_5_;
        t_45_ = t_45_*inv_5_;
        t_46_ = t_46_*inv_5_;
        t_47_ = t_35_*t_28_;
        t_48_ = t_35_*t_29_-t_31_*t_33_;
        t_49_ = t_35_*t_30_-t_31_*t_34_;
        t_50_ = t_35_*t_32_-t_31_*t_36_;
        inv_6_ = safeinv(max(max(max(abs(t_47_), abs(t_48_)), abs(t_49_)), abs(t_50_)));
        t_47_ = t_47_*inv_6_;
        t_48_ = t_48_*inv_6_;
        t_49_ = t_49_*inv_6_;
        t_50_ = t_50_*inv_6_;
        t_51_ = t_35_*a_44_-a_47_*t_33_;
        t_52_ = t_35_*a_45_;
        t_53_ = t_35_*a_46_-a_47_*t_34_;
        t_54_ = t_35_*_pp_var_I5[i_]-a_47_*t_36_;
        inv_7_ = safeinv(max(max(max(abs(t_51_), abs(t_52_)), abs(t_53_)), abs(t_54_)));
        t_51_ = t_51_*inv_7_;
        t_52_ = t_52_*inv_7_;
        t_53_ = t_53_*inv_7_;
        t_54_ = t_54_*inv_7_;
        t_55_ = t_35_*a_51_-a_52_*t_33_;
        t_56_ =  -(a_52_*t_34_);
        t_57_ = t_35_*a_53_;
        t_58_ = t_35_*_pp_var_O[i_]-a_52_*t_36_;
        inv_8_ = safeinv(max(max(max(abs(t_55_), abs(t_56_)), abs(t_57_)), abs(t_58_)));
        t_55_ = t_55_*inv_8_;
        t_56_ = t_56_*inv_8_;
        t_57_ = t_57_*inv_8_;
        t_58_ = t_58_*inv_8_;
        t_59_ = a_14_*t_37_-t_38_*a_13_;
        t_60_ = a_14_*t_39_-t_38_*a_15_;
        t_61_ = a_14_*t_40_;
        t_62_ = a_14_*t_41_;
        t_63_ = a_14_*t_42_-t_38_*a_16_;
        t_64_ = a_14_*t_43_;
        t_65_ = a_14_*t_44_;
        t_66_ = a_14_*t_45_;
        t_67_ = a_14_*t_46_-t_38_*_pp_var_C2[i_];
        inv_9_ = safeinv(max(max(max(max(max(max(max(max(abs(t_59_), abs(t_60_)), abs(t_61_)), abs(t_62_)), abs(t_63_)), abs(t_64_)), abs(t_65_)), abs(t_66_)), abs(t_67_)));
        t_59_ = t_59_*inv_9_;
        t_60_ = t_60_*inv_9_;
        t_61_ = t_61_*inv_9_;
        t_62_ = t_62_*inv_9_;
        t_63_ = t_63_*inv_9_;
        t_64_ = t_64_*inv_9_;
        t_65_ = t_65_*inv_9_;
        t_66_ = t_66_*inv_9_;
        t_67_ = t_67_*inv_9_;
        t_68_ =  -(a_17_*a_13_);
        t_69_ = a_14_*a_18_-a_17_*a_15_;
        t_70_ = a_14_*a_19_;
        t_71_ =  -(a_17_*a_16_);
        t_72_ = a_14_*a_20_;
        t_73_ = a_14_*_pp_var_C3[i_]-a_17_*_pp_var_C2[i_];
        inv_10_ = safeinv(max(max(max(max(max(abs(t_68_), abs(t_69_)), abs(t_70_)), abs(t_71_)), abs(t_72_)), abs(t_73_)));
        t_68_ = t_68_*inv_10_;
        t_69_ = t_69_*inv_10_;
        t_70_ = t_70_*inv_10_;
        t_71_ = t_71_*inv_10_;
        t_72_ = t_72_*inv_10_;
        t_73_ = t_73_*inv_10_;
        t_74_ = a_14_*t_12_-t_13_*a_13_;
        t_75_ =  -(t_13_*a_15_);
        t_76_ = a_14_*t_14_-t_13_*a_16_;
        t_77_ = a_14_*t_15_;
        t_78_ = a_14_*t_16_-t_13_*_pp_var_C2[i_];
        inv_11_ = safeinv(max(max(max(max(abs(t_74_), abs(t_75_)), abs(t_76_)), abs(t_77_)), abs(t_78_)));
        t_74_ = t_74_*inv_11_;
        t_75_ = t_75_*inv_11_;
        t_76_ = t_76_*inv_11_;
        t_77_ = t_77_*inv_11_;
        t_78_ = t_78_*inv_11_;
        t_79_ = t_53_*t_59_;
        t_80_ = t_53_*t_60_;
        t_81_ = t_53_*t_61_;
        t_82_ = t_53_*t_62_-t_66_*t_51_;
        t_83_ = t_53_*t_63_;
        t_84_ = t_53_*t_64_;
        t_85_ = t_53_*t_65_-t_66_*t_52_;
        t_86_ = t_53_*t_67_-t_66_*t_54_;
        inv_12_ = safeinv(max(max(max(max(max(max(max(abs(t_79_), abs(t_80_)), abs(t_81_)), abs(t_82_)), abs(t_83_)), abs(t_84_)), abs(t_85_)), abs(t_86_)));
        t_79_ = t_79_*inv_12_;
        t_80_ = t_80_*inv_12_;
        t_81_ = t_81_*inv_12_;
        t_82_ = t_82_*inv_12_;
        t_83_ = t_83_*inv_12_;
        t_84_ = t_84_*inv_12_;
        t_85_ = t_85_*inv_12_;
        t_86_ = t_86_*inv_12_;
        t_87_ = t_53_*t_47_;
        t_88_ = t_53_*t_48_-t_49_*t_51_;
        t_89_ =  -(t_49_*t_52_);
        t_90_ = t_53_*t_50_-t_49_*t_54_;
        inv_13_ = safeinv(max(max(max(abs(t_87_), abs(t_88_)), abs(t_89_)), abs(t_90_)));
        t_87_ = t_87_*inv_13_;
        t_88_ = t_88_*inv_13_;
        t_89_ = t_89_*inv_13_;
        t_90_ = t_90_*inv_13_;
        t_91_ = t_53_*a_40_;
        t_92_ =  -(a_43_*t_51_);
        t_93_ = t_53_*a_41_;
        t_94_ = t_53_*a_42_-a_43_*t_52_;
        t_95_ = t_53_*_pp_var_I4[i_]-a_43_*t_54_;
        inv_14_ = safeinv(max(max(max(max(abs(t_91_), abs(t_92_)), abs(t_93_)), abs(t_94_)), abs(t_95_)));
        t_91_ = t_91_*inv_14_;
        t_92_ = t_92_*inv_14_;
        t_93_ = t_93_*inv_14_;
        t_94_ = t_94_*inv_14_;
        t_95_ = t_95_*inv_14_;
        t_96_ = t_53_*t_33_-t_34_*t_51_;
        t_97_ =  -(t_34_*t_52_);
        t_98_ = t_53_*t_35_;
        t_99_ = t_53_*t_36_-t_34_*t_54_;
        inv_15_ = safeinv(max(max(max(abs(t_96_), abs(t_97_)), abs(t_98_)), abs(t_99_)));
        t_96_ = t_96_*inv_15_;
        t_97_ = t_97_*inv_15_;
        t_98_ = t_98_*inv_15_;
        t_99_ = t_99_*inv_15_;
        t_100_ = t_53_*t_55_-t_56_*t_51_;
        t_101_ =  -(t_56_*t_52_);
        t_102_ = t_53_*t_57_;
        t_103_ = t_53_*t_58_-t_56_*t_54_;
        inv_16_ = safeinv(max(max(max(abs(t_100_), abs(t_101_)), abs(t_102_)), abs(t_103_)));
        t_100_ = t_100_*inv_16_;
        t_101_ = t_101_*inv_16_;
        t_102_ = t_102_*inv_16_;
        t_103_ = t_103_*inv_16_;
        t_104_ = t_88_*t_79_;
        t_105_ = t_88_*t_80_;
        t_106_ = t_88_*t_81_-t_82_*t_87_;
        t_107_ = t_88_*t_83_;
        t_108_ = t_88_*t_84_;
        t_109_ = t_88_*t_85_-t_82_*t_89_;
        t_110_ = t_88_*t_86_-t_82_*t_90_;
        inv_17_ = safeinv(max(max(max(max(max(max(abs(t_104_), abs(t_105_)), abs(t_106_)), abs(t_107_)), abs(t_108_)), abs(t_109_)), abs(t_110_)));
        t_104_ = t_104_*inv_17_;
        t_105_ = t_105_*inv_17_;
        t_106_ = t_106_*inv_17_;
        t_107_ = t_107_*inv_17_;
        t_108_ = t_108_*inv_17_;
        t_109_ = t_109_*inv_17_;
        t_110_ = t_110_*inv_17_;
        t_111_ = t_88_*a_21_;
        t_112_ = t_88_*a_22_-a_23_*t_87_;
        t_113_ = t_88_*a_24_-a_23_*t_89_;
        t_114_ = t_88_*_pp_var_C4[i_]-a_23_*t_90_;
        inv_18_ = safeinv(max(max(max(abs(t_111_), abs(t_112_)), abs(t_113_)), abs(t_114_)));
        t_111_ = t_111_*inv_18_;
        t_112_ = t_112_*inv_18_;
        t_113_ = t_113_*inv_18_;
        t_114_ = t_114_*inv_18_;
        t_115_ = t_88_*t_91_-t_92_*t_87_;
        t_116_ = t_88_*t_93_;
        t_117_ = t_88_*t_94_-t_92_*t_89_;
        t_118_ = t_88_*t_95_-t_92_*t_90_;
        inv_19_ = safeinv(max(max(max(abs(t_115_), abs(t_116_)), abs(t_117_)), abs(t_118_)));
        t_115_ = t_115_*inv_19_;
        t_116_ = t_116_*inv_19_;
        t_117_ = t_117_*inv_19_;
        t_118_ = t_118_*inv_19_;
        t_119_ =  -(t_51_*t_87_);
        t_120_ = t_88_*t_52_-t_51_*t_89_;
        t_121_ = t_88_*t_53_;
        t_122_ = t_88_*t_54_-t_51_*t_90_;
        inv_20_ = safeinv(max(max(max(abs(t_119_), abs(t_120_)), abs(t_121_)), abs(t_122_)));
        t_119_ = t_119_*inv_20_;
        t_120_ = t_120_*inv_20_;
        t_121_ = t_121_*inv_20_;
        t_122_ = t_122_*inv_20_;
        t_123_ =  -(t_96_*t_87_);
        t_124_ = t_88_*t_97_-t_96_*t_89_;
        t_125_ = t_88_*t_98_;
        t_126_ = t_88_*t_99_-t_96_*t_90_;
        inv_21_ = safeinv(max(max(max(abs(t_123_), abs(t_124_)), abs(t_125_)), abs(t_126_)));
        t_123_ = t_123_*inv_21_;
        t_124_ = t_124_*inv_21_;
        t_125_ = t_125_*inv_21_;
        t_126_ = t_126_*inv_21_;
        t_127_ =  -(t_100_*t_87_);
        t_128_ = t_88_*t_101_-t_100_*t_89_;
        t_129_ = t_88_*t_102_;
        t_130_ = t_88_*t_103_-t_100_*t_90_;
        inv_22_ = safeinv(max(max(max(abs(t_127_), abs(t_128_)), abs(t_129_)), abs(t_130_)));
        t_127_ = t_127_*inv_22_;
        t_128_ = t_128_*inv_22_;
        t_129_ = t_129_*inv_22_;
        t_130_ = t_130_*inv_22_;
        t_131_ = a_38_*t_104_;
        t_132_ = a_38_*t_105_-t_108_*a_36_;
        t_133_ = a_38_*t_106_;
        t_134_ = a_38_*t_107_-t_108_*a_37_;
        t_135_ = a_38_*t_109_-t_108_*a_39_;
        t_136_ = a_38_*t_110_-t_108_*_pp_var_I3[i_];
        inv_23_ = safeinv(max(max(max(max(max(abs(t_131_), abs(t_132_)), abs(t_133_)), abs(t_134_)), abs(t_135_)), abs(t_136_)));
        t_131_ = t_131_*inv_23_;
        t_132_ = t_132_*inv_23_;
        t_133_ = t_133_*inv_23_;
        t_134_ = t_134_*inv_23_;
        t_135_ = t_135_*inv_23_;
        t_136_ = t_136_*inv_23_;
        t_137_ = a_38_*t_68_;
        t_138_ = a_38_*t_69_-t_72_*a_36_;
        t_139_ = a_38_*t_70_;
        t_140_ = a_38_*t_71_-t_72_*a_37_;
        t_141_ =  -(t_72_*a_39_);
        t_142_ = a_38_*t_73_-t_72_*_pp_var_I3[i_];
        inv_24_ = safeinv(max(max(max(max(max(abs(t_137_), abs(t_138_)), abs(t_139_)), abs(t_140_)), abs(t_141_)), abs(t_142_)));
        t_137_ = t_137_*inv_24_;
        t_138_ = t_138_*inv_24_;
        t_139_ = t_139_*inv_24_;
        t_140_ = t_140_*inv_24_;
        t_141_ = t_141_*inv_24_;
        t_142_ = t_142_*inv_24_;
        t_143_ = a_38_*t_74_;
        t_144_ = a_38_*t_75_-t_77_*a_36_;
        t_145_ = a_38_*t_76_-t_77_*a_37_;
        t_146_ =  -(t_77_*a_39_);
        t_147_ = a_38_*t_78_-t_77_*_pp_var_I3[i_];
        inv_25_ = safeinv(max(max(max(max(abs(t_143_), abs(t_144_)), abs(t_145_)), abs(t_146_)), abs(t_147_)));
        t_143_ = t_143_*inv_25_;
        t_144_ = t_144_*inv_25_;
        t_145_ = t_145_*inv_25_;
        t_146_ = t_146_*inv_25_;
        t_147_ = t_147_*inv_25_;
        t_148_ =  -(t_116_*a_36_);
        t_149_ = a_38_*t_115_;
        t_150_ =  -(t_116_*a_37_);
        t_151_ = a_38_*t_117_-t_116_*a_39_;
        t_152_ = a_38_*t_118_-t_116_*_pp_var_I3[i_];
        inv_26_ = safeinv(max(max(max(max(abs(t_148_), abs(t_149_)), abs(t_150_)), abs(t_151_)), abs(t_152_)));
        t_148_ = t_148_*inv_26_;
        t_149_ = t_149_*inv_26_;
        t_150_ = t_150_*inv_26_;
        t_151_ = t_151_*inv_26_;
        t_152_ = t_152_*inv_26_;
        t_153_ = t_112_*t_131_;
        t_154_ = t_112_*t_132_-t_133_*t_111_;
        t_155_ = t_112_*t_134_;
        t_156_ = t_112_*t_135_-t_133_*t_113_;
        t_157_ = t_112_*t_136_-t_133_*t_114_;
        inv_27_ = safeinv(max(max(max(max(abs(t_153_), abs(t_154_)), abs(t_155_)), abs(t_156_)), abs(t_157_)));
        t_153_ = t_153_*inv_27_;
        t_154_ = t_154_*inv_27_;
        t_155_ = t_155_*inv_27_;
        t_156_ = t_156_*inv_27_;
        t_157_ = t_157_*inv_27_;
        t_158_ = t_112_*t_137_;
        t_159_ = t_112_*t_138_-t_139_*t_111_;
        t_160_ = t_112_*t_140_;
        t_161_ = t_112_*t_141_-t_139_*t_113_;
        t_162_ = t_112_*t_142_-t_139_*t_114_;
        inv_28_ = safeinv(max(max(max(max(abs(t_158_), abs(t_159_)), abs(t_160_)), abs(t_161_)), abs(t_162_)));
        t_158_ = t_158_*inv_28_;
        t_159_ = t_159_*inv_28_;
        t_160_ = t_160_*inv_28_;
        t_161_ = t_161_*inv_28_;
        t_162_ = t_162_*inv_28_;
        t_163_ =  -(t_87_*t_111_);
        t_164_ = t_112_*t_88_;
        t_165_ = t_112_*t_89_-t_87_*t_113_;
        t_166_ = t_112_*t_90_-t_87_*t_114_;
        inv_29_ = safeinv(max(max(max(abs(t_163_), abs(t_164_)), abs(t_165_)), abs(t_166_)));
        t_163_ = t_163_*inv_29_;
        t_164_ = t_164_*inv_29_;
        t_165_ = t_165_*inv_29_;
        t_166_ = t_166_*inv_29_;
        t_167_ = t_112_*t_148_-t_149_*t_111_;
        t_168_ = t_112_*t_150_;
        t_169_ = t_112_*t_151_-t_149_*t_113_;
        t_170_ = t_112_*t_152_-t_149_*t_114_;
        inv_30_ = safeinv(max(max(max(abs(t_167_), abs(t_168_)), abs(t_169_)), abs(t_170_)));
        t_167_ = t_167_*inv_30_;
        t_168_ = t_168_*inv_30_;
        t_169_ = t_169_*inv_30_;
        t_170_ = t_170_*inv_30_;
        t_171_ =  -(t_119_*t_111_);
        t_172_ = t_112_*t_120_-t_119_*t_113_;
        t_173_ = t_112_*t_121_;
        t_174_ = t_112_*t_122_-t_119_*t_114_;
        inv_31_ = safeinv(max(max(max(abs(t_171_), abs(t_172_)), abs(t_173_)), abs(t_174_)));
        t_171_ = t_171_*inv_31_;
        t_172_ = t_172_*inv_31_;
        t_173_ = t_173_*inv_31_;
        t_174_ = t_174_*inv_31_;
        t_175_ =  -(t_123_*t_111_);
        t_176_ = t_112_*t_124_-t_123_*t_113_;
        t_177_ = t_112_*t_125_;
        t_178_ = t_112_*t_126_-t_123_*t_114_;
        inv_32_ = safeinv(max(max(max(abs(t_175_), abs(t_176_)), abs(t_177_)), abs(t_178_)));
        t_175_ = t_175_*inv_32_;
        t_176_ = t_176_*inv_32_;
        t_177_ = t_177_*inv_32_;
        t_178_ = t_178_*inv_32_;
        t_179_ =  -(t_127_*t_111_);
        t_180_ = t_112_*t_128_-t_127_*t_113_;
        t_181_ = t_112_*t_129_;
        t_182_ = t_112_*t_130_-t_127_*t_114_;
        inv_33_ = safeinv(max(max(max(abs(t_179_), abs(t_180_)), abs(t_181_)), abs(t_182_)));
        t_179_ = t_179_*inv_33_;
        t_180_ = t_180_*inv_33_;
        t_181_ = t_181_*inv_33_;
        t_182_ = t_182_*inv_33_;
        t_183_ = t_153_*a_14_;
        t_184_ = t_153_*a_15_-a_13_*t_154_;
        t_185_ = t_153_*a_16_-a_13_*t_155_;
        t_186_ =  -(a_13_*t_156_);
        t_187_ = t_153_*_pp_var_C2[i_]-a_13_*t_157_;
        inv_34_ = safeinv(max(max(max(max(abs(t_183_), abs(t_184_)), abs(t_185_)), abs(t_186_)), abs(t_187_)));
        t_183_ = t_183_*inv_34_;
        t_184_ = t_184_*inv_34_;
        t_185_ = t_185_*inv_34_;
        t_186_ = t_186_*inv_34_;
        t_187_ = t_187_*inv_34_;
        t_188_ = t_153_*t_159_-t_158_*t_154_;
        t_189_ = t_153_*t_160_-t_158_*t_155_;
        t_190_ = t_153_*t_161_-t_158_*t_156_;
        t_191_ = t_153_*t_162_-t_158_*t_157_;
        inv_35_ = safeinv(max(max(max(abs(t_188_), abs(t_189_)), abs(t_190_)), abs(t_191_)));
        t_188_ = t_188_*inv_35_;
        t_189_ = t_189_*inv_35_;
        t_190_ = t_190_*inv_35_;
        t_191_ = t_191_*inv_35_;
        t_192_ =  -(a_29_*t_154_);
        t_193_ = t_153_*a_30_;
        t_194_ = t_153_*a_31_-a_29_*t_155_;
        t_195_ =  -(a_29_*t_156_);
        t_196_ = t_153_*_pp_var_I1[i_]-a_29_*t_157_;
        inv_36_ = safeinv(max(max(max(max(abs(t_192_), abs(t_193_)), abs(t_194_)), abs(t_195_)), abs(t_196_)));
        t_192_ = t_192_*inv_36_;
        t_193_ = t_193_*inv_36_;
        t_194_ = t_194_*inv_36_;
        t_195_ = t_195_*inv_36_;
        t_196_ = t_196_*inv_36_;
        t_197_ = t_153_*t_144_-t_143_*t_154_;
        t_198_ = t_153_*t_145_-t_143_*t_155_;
        t_199_ = t_153_*t_146_-t_143_*t_156_;
        t_200_ = t_153_*t_147_-t_143_*t_157_;
        inv_37_ = safeinv(max(max(max(abs(t_197_), abs(t_198_)), abs(t_199_)), abs(t_200_)));
        t_197_ = t_197_*inv_37_;
        t_198_ = t_198_*inv_37_;
        t_199_ = t_199_*inv_37_;
        t_200_ = t_200_*inv_37_;
        t_201_ = t_198_*t_153_;
        t_202_ = t_198_*t_154_-t_155_*t_197_;
        t_203_ = t_198_*t_156_-t_155_*t_199_;
        t_204_ = t_198_*t_157_-t_155_*t_200_;
        inv_38_ = safeinv(max(max(max(abs(t_201_), abs(t_202_)), abs(t_203_)), abs(t_204_)));
        t_201_ = t_201_*inv_38_;
        t_202_ = t_202_*inv_38_;
        t_203_ = t_203_*inv_38_;
        t_204_ = t_204_*inv_38_;
        t_205_ = t_198_*t_183_;
        t_206_ = t_198_*t_184_-t_185_*t_197_;
        t_207_ = t_198_*t_186_-t_185_*t_199_;
        t_208_ = t_198_*t_187_-t_185_*t_200_;
        inv_39_ = safeinv(max(max(max(abs(t_205_), abs(t_206_)), abs(t_207_)), abs(t_208_)));
        t_205_ = t_205_*inv_39_;
        t_206_ = t_206_*inv_39_;
        t_207_ = t_207_*inv_39_;
        t_208_ = t_208_*inv_39_;
        t_209_ = t_198_*t_188_-t_189_*t_197_;
        t_210_ = t_198_*t_190_-t_189_*t_199_;
        t_211_ = t_198_*t_191_-t_189_*t_200_;
        inv_40_ = safeinv(max(max(abs(t_209_), abs(t_210_)), abs(t_211_)));
        t_209_ = t_209_*inv_40_;
        t_210_ = t_210_*inv_40_;
        t_211_ = t_211_*inv_40_;
        t_212_ = t_198_*t_192_-t_194_*t_197_;
        t_213_ = t_198_*t_193_;
        t_214_ = t_198_*t_195_-t_194_*t_199_;
        t_215_ = t_198_*t_196_-t_194_*t_200_;
        inv_41_ = safeinv(max(max(max(abs(t_212_), abs(t_213_)), abs(t_214_)), abs(t_215_)));
        t_212_ = t_212_*inv_41_;
        t_213_ = t_213_*inv_41_;
        t_214_ = t_214_*inv_41_;
        t_215_ = t_215_*inv_41_;
        t_216_ = t_198_*a_36_-a_37_*t_197_;
        t_217_ = t_198_*a_38_;
        t_218_ = t_198_*a_39_-a_37_*t_199_;
        t_219_ = t_198_*_pp_var_I3[i_]-a_37_*t_200_;
        inv_42_ = safeinv(max(max(max(abs(t_216_), abs(t_217_)), abs(t_218_)), abs(t_219_)));
        t_216_ = t_216_*inv_42_;
        t_217_ = t_217_*inv_42_;
        t_218_ = t_218_*inv_42_;
        t_219_ = t_219_*inv_42_;
        t_220_ = t_198_*t_167_-t_168_*t_197_;
        t_221_ = t_198_*t_169_-t_168_*t_199_;
        t_222_ = t_198_*t_170_-t_168_*t_200_;
        inv_43_ = safeinv(max(max(abs(t_220_), abs(t_221_)), abs(t_222_)));
        t_220_ = t_220_*inv_43_;
        t_221_ = t_221_*inv_43_;
        t_222_ = t_222_*inv_43_;
        t_223_ = t_221_*t_201_;
        t_224_ = t_221_*t_202_-t_203_*t_220_;
        t_225_ = t_221_*t_204_-t_203_*t_222_;
        inv_44_ = safeinv(max(max(abs(t_223_), abs(t_224_)), abs(t_225_)));
        t_223_ = t_223_*inv_44_;
        t_224_ = t_224_*inv_44_;
        t_225_ = t_225_*inv_44_;
        t_226_ = t_221_*t_205_;
        t_227_ = t_221_*t_206_-t_207_*t_220_;
        t_228_ = t_221_*t_208_-t_207_*t_222_;
        inv_45_ = safeinv(max(max(abs(t_226_), abs(t_227_)), abs(t_228_)));
        t_226_ = t_226_*inv_45_;
        t_227_ = t_227_*inv_45_;
        t_228_ = t_228_*inv_45_;
        t_229_ = t_221_*t_209_-t_210_*t_220_;
        t_230_ = t_221_*t_211_-t_210_*t_222_;
        inv_46_ = safeinv(max(abs(t_229_), abs(t_230_)));
        t_229_ = t_229_*inv_46_;
        t_230_ = t_230_*inv_46_;
        t_231_ = t_221_*t_111_-t_113_*t_220_;
        t_232_ = t_221_*t_112_;
        t_233_ = t_221_*t_114_-t_113_*t_222_;
        inv_47_ = safeinv(max(max(abs(t_231_), abs(t_232_)), abs(t_233_)));
        t_231_ = t_231_*inv_47_;
        t_232_ = t_232_*inv_47_;
        t_233_ = t_233_*inv_47_;
        t_234_ = t_221_*t_163_-t_165_*t_220_;
        t_235_ = t_221_*t_164_;
        t_236_ = t_221_*t_166_-t_165_*t_222_;
        inv_48_ = safeinv(max(max(abs(t_234_), abs(t_235_)), abs(t_236_)));
        t_234_ = t_234_*inv_48_;
        t_235_ = t_235_*inv_48_;
        t_236_ = t_236_*inv_48_;
        t_237_ = t_221_*t_212_-t_214_*t_220_;
        t_238_ = t_221_*t_213_;
        t_239_ = t_221_*t_215_-t_214_*t_222_;
        inv_49_ = safeinv(max(max(abs(t_237_), abs(t_238_)), abs(t_239_)));
        t_237_ = t_237_*inv_49_;
        t_238_ = t_238_*inv_49_;
        t_239_ = t_239_*inv_49_;
        t_240_ = t_221_*t_197_-t_199_*t_220_;
        t_241_ = t_221_*t_198_;
        t_242_ = t_221_*t_200_-t_199_*t_222_;
        inv_50_ = safeinv(max(max(abs(t_240_), abs(t_241_)), abs(t_242_)));
        t_240_ = t_240_*inv_50_;
        t_241_ = t_241_*inv_50_;
        t_242_ = t_242_*inv_50_;
        t_243_ = t_221_*t_216_-t_218_*t_220_;
        t_244_ = t_221_*t_217_;
        t_245_ = t_221_*t_219_-t_218_*t_222_;
        inv_51_ = safeinv(max(max(abs(t_243_), abs(t_244_)), abs(t_245_)));
        t_243_ = t_243_*inv_51_;
        t_244_ = t_244_*inv_51_;
        t_245_ = t_245_*inv_51_;
        t_246_ = t_221_*t_171_-t_172_*t_220_;
        t_247_ = t_221_*t_173_;
        t_248_ = t_221_*t_174_-t_172_*t_222_;
        inv_52_ = safeinv(max(max(abs(t_246_), abs(t_247_)), abs(t_248_)));
        t_246_ = t_246_*inv_52_;
        t_247_ = t_247_*inv_52_;
        t_248_ = t_248_*inv_52_;
        t_249_ = t_221_*t_175_-t_176_*t_220_;
        t_250_ = t_221_*t_177_;
        t_251_ = t_221_*t_178_-t_176_*t_222_;
        inv_53_ = safeinv(max(max(abs(t_249_), abs(t_250_)), abs(t_251_)));
        t_249_ = t_249_*inv_53_;
        t_250_ = t_250_*inv_53_;
        t_251_ = t_251_*inv_53_;
        t_252_ = t_221_*t_179_-t_180_*t_220_;
        t_253_ = t_221_*t_181_;
        t_254_ = t_221_*t_182_-t_180_*t_222_;
        inv_54_ = safeinv(max(max(abs(t_252_), abs(t_253_)), abs(t_254_)));
        t_252_ = t_252_*inv_54_;
        t_253_ = t_253_*inv_54_;
        t_254_ = t_254_*inv_54_;
        t_255_ = t_229_*t_223_;
        t_256_ = t_229_*t_225_-t_224_*t_230_;
        inv_55_ = safeinv(max(abs(t_255_), abs(t_256_)));
        t_255_ = t_255_*inv_55_;
        t_256_ = t_256_*inv_55_;
        t_257_ = t_229_*t_226_;
        t_258_ = t_229_*t_228_-t_227_*t_230_;
        inv_56_ = safeinv(max(abs(t_257_), abs(t_258_)));
        t_257_ = t_257_*inv_56_;
        t_258_ = t_258_*inv_56_;
        t_259_ = t_229_*t_232_;
        t_260_ = t_229_*t_233_-t_231_*t_230_;
        inv_57_ = safeinv(max(abs(t_259_), abs(t_260_)));
        t_259_ = t_259_*inv_57_;
        t_260_ = t_260_*inv_57_;
        t_261_ = t_229_*t_235_;
        t_262_ = t_229_*t_236_-t_234_*t_230_;
        inv_58_ = safeinv(max(abs(t_261_), abs(t_262_)));
        t_261_ = t_261_*inv_58_;
        t_262_ = t_262_*inv_58_;
        t_263_ = t_229_*t_238_;
        t_264_ = t_229_*t_239_-t_237_*t_230_;
        inv_59_ = safeinv(max(abs(t_263_), abs(t_264_)));
        t_263_ = t_263_*inv_59_;
        t_264_ = t_264_*inv_59_;
        t_265_ = t_229_*t_241_;
        t_266_ = t_229_*t_242_-t_240_*t_230_;
        inv_60_ = safeinv(max(abs(t_265_), abs(t_266_)));
        t_265_ = t_265_*inv_60_;
        t_266_ = t_266_*inv_60_;
        t_267_ = t_229_*t_244_;
        t_268_ = t_229_*t_245_-t_243_*t_230_;
        inv_61_ = safeinv(max(abs(t_267_), abs(t_268_)));
        t_267_ = t_267_*inv_61_;
        t_268_ = t_268_*inv_61_;
        t_269_ = t_229_*t_221_;
        t_270_ = t_229_*t_222_-t_220_*t_230_;
        inv_62_ = safeinv(max(abs(t_269_), abs(t_270_)));
        t_269_ = t_269_*inv_62_;
        t_270_ = t_270_*inv_62_;
        t_271_ = t_229_*t_247_;
        t_272_ = t_229_*t_248_-t_246_*t_230_;
        inv_63_ = safeinv(max(abs(t_271_), abs(t_272_)));
        t_271_ = t_271_*inv_63_;
        t_272_ = t_272_*inv_63_;
        t_273_ = t_229_*t_250_;
        t_274_ = t_229_*t_251_-t_249_*t_230_;
        inv_64_ = safeinv(max(abs(t_273_), abs(t_274_)));
        t_273_ = t_273_*inv_64_;
        t_274_ = t_274_*inv_64_;
        t_275_ = t_229_*t_253_;
        t_276_ = t_229_*t_254_-t_252_*t_230_;
        inv_65_ = safeinv(max(abs(t_275_), abs(t_276_)));
        t_275_ = t_275_*inv_65_;
        t_276_ = t_276_*inv_65_;
        _pp_var_C1[i_] = t_256_/t_255_;
        _pp_var_C2[i_] = t_258_/t_257_;
        _pp_var_C3[i_] = t_230_/t_229_;
        _pp_var_C4[i_] = t_260_/t_259_;
        _pp_var_C5[i_] = t_262_/t_261_;
        _pp_var_I1[i_] = t_264_/t_263_;
        _pp_var_I2[i_] = t_266_/t_265_;
        _pp_var_I3[i_] = t_268_/t_267_;
        _pp_var_I4[i_] = t_270_/t_269_;
        _pp_var_I5[i_] = t_272_/t_271_;
        _pp_var_I6[i_] = t_274_/t_273_;
        _pp_var_O[i_] = t_276_/t_275_;
    }
}

static void compute_currents(arb_mechanism_ppack* pp) {
    PPACK_IFACE_BLOCK;
    for (arb_size_type i_ = 0; i_ < _pp_var_width; ++i_) {
        auto ion_na_indexi_ = _pp_var_ion_na_index[i_];
        auto node_indexi_ = _pp_var_node_index[i_];
        arb_value_type current_ = 0;
        arb_value_type conductivity_ = 0;
        arb_value_type conductivity_na_ = 0;
        arb_value_type ena = _pp_var_ion_na.reversal_potential[ion_na_indexi_];
        arb_value_type v = _pp_var_vec_v[node_indexi_];
        arb_value_type ina = 0;
        ina = _pp_var_gbar[i_]*_pp_var_O[i_]*(v-ena);
        conductivity_na_ = _pp_var_gbar[i_]*_pp_var_O[i_];
        current_ = ina;
        conductivity_ = _pp_var_gbar[i_]*_pp_var_O[i_];
        _pp_var_vec_i[node_indexi_] = fma(10.0*_pp_var_weight[i_], current_, _pp_var_vec_i[node_indexi_]);
        _pp_var_vec_g[node_indexi_] = fma(10.0*_pp_var_weight[i_], conductivity_, _pp_var_vec_g[node_indexi_]);
        _pp_var_ion_na.conductivity[ion_na_indexi_] = fma(10.0*_pp_var_weight[i_], conductivity_na_, _pp_var_ion_na.conductivity[ion_na_indexi_]);
        _pp_var_ion_na.current_density[ion_na_indexi_] = fma(10.0*_pp_var_weight[i_], ina, _pp_var_ion_na.current_density[ion_na_indexi_]);
    }
}

static void write_ions(arb_mechanism_ppack* pp) {
}

static void apply_events(arb_mechanism_ppack*, arb_deliverable_event_stream*) {}

static void post_event(arb_mechanism_ppack*) {}
#undef PPACK_IFACE_BLOCK
} // namespace kernel_NaV
} // namespace allen_catalogue
} // namespace arb

extern "C" {
  arb_mechanism_interface* make_arb_allen_catalogue_NaV_interface_multicore() {
    static arb_mechanism_interface result;
    result.partition_width = arb::allen_catalogue::kernel_NaV::simd_width_;
    result.backend = arb_backend_kind_cpu;
    result.alignment = arb::allen_catalogue::kernel_NaV::min_align_;
    result.init_mechanism = arb::allen_catalogue::kernel_NaV::init;
    result.compute_currents = arb::allen_catalogue::kernel_NaV::compute_currents;
    result.apply_events = arb::allen_catalogue::kernel_NaV::apply_events;
    result.advance_state = arb::allen_catalogue::kernel_NaV::advance_state;
    result.write_ions = arb::allen_catalogue::kernel_NaV::write_ions;
    result.post_event = arb::allen_catalogue::kernel_NaV::post_event;
    return &result;
  }
}

