#include <algorithm>
#include <cmath>
#include <cstddef>
#include <memory>
#include <arbor/mechanism_abi.h>
#include <arbor/math.hpp>

namespace arb {
namespace default_catalogue {
namespace kernel_v_clamp {

using ::arb::math::exprelr;
using ::arb::math::safeinv;
using ::std::abs;
using ::std::cos;
using ::std::exp;
using ::std::max;
using ::std::min;
using ::std::pow;
using ::std::sin;
using ::std::sqrt;
using ::std::tanh;

static constexpr unsigned simd_width_ = 1;
static constexpr unsigned min_align_ = std::max(alignof(arb_value_type), alignof(arb_index_type));
using ::std::log;

#define PPACK_IFACE_BLOCK \
[[maybe_unused]] auto _pp_var_width                                                 = pp->width;\
[[maybe_unused]] auto _pp_var_n_detectors                                           = pp->n_detectors;\
[[maybe_unused]] auto _pp_var_dt                                                    = pp->dt;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_vec_ci                       = pp->vec_ci;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_v                        = pp->vec_v;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_i                        = pp->vec_i;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_vec_g                        = pp->vec_g;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_temperature_degC             = pp->temperature_degC;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_diam_um                      = pp->diam_um;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_area_um2                     = pp->area_um2;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_time_since_spike             = pp->time_since_spike;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_node_index                   = pp->node_index;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_peer_index                   = pp->peer_index;\
[[maybe_unused]] arb_index_type * __restrict__ _pp_var_multiplicity                 = pp->multiplicity;\
[[maybe_unused]] arb_value_type * __restrict__ _pp_var_weight                       = pp->weight;\
[[maybe_unused]] auto& _pp_var_events                                               = pp->events;\
[[maybe_unused]] auto _pp_var_mechanism_id                                          = pp->mechanism_id;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_contiguous               = pp->index_constraints.n_contiguous;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_constant                 = pp->index_constraints.n_constant;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_independent              = pp->index_constraints.n_independent;\
[[maybe_unused]] arb_size_type _pp_var_index_constraints_n_none                     = pp->index_constraints.n_none;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_contiguous  = pp->index_constraints.contiguous;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_constant    = pp->index_constraints.constant;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_independent = pp->index_constraints.independent;\
[[maybe_unused]] arb_index_type* __restrict__ _pp_var_index_constraints_none        = pp->index_constraints.none;\
[[maybe_unused]] auto _pp_var_v0 = pp->globals[0];\
[[maybe_unused]] auto const * const * _pp_var_random_numbers = pp->random_numbers;\
//End of IFACEBLOCK


// interface methods
static void init(arb_mechanism_ppack* pp) {
}

static void advance_state(arb_mechanism_ppack* pp) {
}

static void compute_currents(arb_mechanism_ppack* pp) {
    PPACK_IFACE_BLOCK;
    for (arb_size_type i_ = 0; i_ < _pp_var_width; ++i_) {
        auto node_indexi_ = _pp_var_node_index[i_];
        arb_value_type v = _pp_var_vec_v[node_indexi_];
        v = _pp_var_v0;
        _pp_var_vec_v[node_indexi_] = v;
    }
}

static void write_ions(arb_mechanism_ppack* pp) {
}

static void apply_events(arb_mechanism_ppack*, arb_deliverable_event_stream*) {}

static void post_event(arb_mechanism_ppack*) {}
#undef PPACK_IFACE_BLOCK
} // namespace kernel_v_clamp
} // namespace default_catalogue
} // namespace arb

extern "C" {
  arb_mechanism_interface* make_arb_default_catalogue_v_clamp_interface_multicore() {
    static arb_mechanism_interface result;
    result.partition_width = arb::default_catalogue::kernel_v_clamp::simd_width_;
    result.backend = arb_backend_kind_cpu;
    result.alignment = arb::default_catalogue::kernel_v_clamp::min_align_;
    result.init_mechanism = arb::default_catalogue::kernel_v_clamp::init;
    result.compute_currents = arb::default_catalogue::kernel_v_clamp::compute_currents;
    result.apply_events = arb::default_catalogue::kernel_v_clamp::apply_events;
    result.advance_state = arb::default_catalogue::kernel_v_clamp::advance_state;
    result.write_ions = arb::default_catalogue::kernel_v_clamp::write_ions;
    result.post_event = arb::default_catalogue::kernel_v_clamp::post_event;
    return &result;
  }
}

